#include "hardware_t.h"
#include "tc_vars.h"
#include "flash_t.h"
#include "display_t.h"
//#include "math.h"

// Class Display
//##########################################################################################################################################################

void Display::DrawInitialScreen(void)
{
	unsigned long lX;

	//Top area above the grid
	for (lX = TOP_PLANE_MIN; lX < TOP_PLANE_MAX; lX++)
	{
		Buffer_UI2Plane[lX] = *(UI_Plane2_Flash + lX);
		UI_Plane3[lX] = *(UI_Plane3_Flash + lX);
	}


// BF delete old cursors and markers
	//Grid area
	for (lX = GRID_PLANE_MIN; lX < GRID_PLANE_MAX; lX++)
	{
		Buffer_Mark1Plane[lX] = 0;
		Marker_Plane1[lX] = 0;

	}	

	//Bottom area
	for (lX = BOTT_PLANE_MIN; lX < BOTT_PLANE_MAX; lX++)
	{
		UI_Plane1[lX] = *(UI_Plane1_Flash + lX);
		Buffer_UI2Plane[lX] = *(UI_Plane2_Flash + lX);		
		UI_Plane3[lX] = *(UI_Plane3_Flash + lX);
		UI_Plane4[lX] = *(UI_Plane4_Flash + lX);
		UI_Plane5[lX] = *(UI_Plane5_Flash + lX);
	}

	//------------------------------------------------------------------------------
	//Draw Grids		//BF
	//------------------------------------------------------------------------------
	if (XY_Mode)								//Check if XY-mode
	GRIDXY(GRID_XY_XOFFS, GRID_YOFFS, Grid_Plane);				// Draw XY grid into gridplane
	else if (MenuStatus[MENU_TIMEBASE][1] == 1) 				//delayed mode
	GRIDDELAYED(GRID_XOFFS, GRID_YOFFS, Grid_Plane);
	else if (FFT_Mode != FFT_OFF) 						// FFT mode
	GRIDFFT(GRID_XOFFS_FFT, GRID_YOFFS_FFT, Grid_Plane);			// Draw FFT grid into gridplane
	else 									//main mode
	GRID(GRID_XOFFS, GRID_YOFFS, Grid_Plane);				// Draw main grid into gridplane
	
	
	//Print text into the top area
	StatusUpdate();

	FFT_DrawStatus(0);	// draw FFT status values

}
//##########################################################################################################################################################

// BF changed (complete grid changed, new subdivisions and new grid dimension GRID_WIDTH 600 x 400)
void Display::GRID(unsigned long x, unsigned long y, unsigned long *layer) 		// Draw Grid subroutine
{
	unsigned long lX;
	unsigned long lY;
	
	
	// Clear Gridplane
	for(lX =  GRID_PLANE_MIN; lX < GRID_PLANE_MAX; lX++)
	{ layer[lX] = 0; }	


	//---------------------------------------------------------------------------------------------
	// Horizontal (X) division from left to right - 12 main divs and 5 sub divs
	//---------------------------------------------------------------------------------------------
	for (lX = x; lX < (x + GRID_WIDTH + 1); lX ++)
	{
		//draw top and bottom edge lines and middle line, 
		PIXELP(lX, y - 1, 1, layer);				// Top Line
		PIXELP(lX, y + GRID_HEIGHT + 1, 1, layer);		// Bottom Line
		//print every second point 
		if ((lX - x)%2 == 0) {
			PIXELP(lX, y + GRID_HEIGHT / 2, 1, layer);	// Middle Line
		}	
		
		
		//draw subdivisions every 10th pixel - one square is 50 pixel wide and has 5 subdivisions
		if( ((lX - x)%10 == 0)&&((lX - x) > 5) && ((lX - x) < (GRID_WIDTH - 5)) && ((lX - x) != (GRID_WIDTH / 2)) && ((lX - x)%50 != 0) )
		{
			//Top 2 pixel line for 5 subdivisions (every 10th x-pixel)
			PIXELP(lX, y + 0, 1, layer);
			PIXELP(lX, y + 2, 1, layer);
			//Middle top 2 pixel line for 5 subdivisions
			PIXELP(lX, y + (GRID_HEIGHT / 2) - 3, 1, layer);
			PIXELP(lX, y + (GRID_HEIGHT / 2) - 1, 1, layer);
			//Middle bottom 2 pixel line for 5 subdivisions
			PIXELP(lX, y + (GRID_HEIGHT / 2) + 1, 1, layer);
			PIXELP(lX, y + (GRID_HEIGHT / 2) + 3, 1, layer);
			//Bottom 2 pixel line for 5 subdivisions
			PIXELP(lX, y + GRID_HEIGHT - 2, 1, layer);
			PIXELP(lX, y + GRID_HEIGHT, 1, layer);
		}	

		//Main division - 50 pixel for one division
		if (((lX - x)%50 == 0) && ((lX - x) > 5) && ((lX - x) < GRID_WIDTH - 5) && ((lX - x) != (GRID_WIDTH / 2)))
		{
			//Top line division
			PIXELP(lX, y + 1, 1, layer);
			PIXELP(lX, y + 3, 1, layer);
			PIXELP(lX, y + 5, 1, layer);
			//Middle line top division
			PIXELP(lX, y + ((GRID_HEIGHT / 2) - 6), 1, layer);
			PIXELP(lX, y + ((GRID_HEIGHT / 2) - 4), 1, layer);
			PIXELP(lX, y + ((GRID_HEIGHT / 2) - 2), 1, layer);
			//Middle line bottom division
			PIXELP(lX, y + ((GRID_HEIGHT / 2) + 2), 1, layer);
			PIXELP(lX, y + ((GRID_HEIGHT / 2) + 4), 1, layer);
			PIXELP(lX, y + ((GRID_HEIGHT / 2) + 6), 1, layer);
			//Bottom line division
			PIXELP(lX, y + (GRID_HEIGHT - 5), 1, layer);
			PIXELP(lX, y + (GRID_HEIGHT - 3), 1, layer);
			PIXELP(lX, y + (GRID_HEIGHT - 1), 1, layer);

			//Grid lines vertical - grid width modulo 50 = 12 lines with one point every 10th pixel
			if (dotted_grid)
				for (lY = y + 10; lY < (y + GRID_HEIGHT - 5); lY += 10)	//dotted line
				{ PIXELP(lX, lY, 1, layer); }
			else
				for (lY = y; lY < (y + GRID_HEIGHT); lY ++)		//solid line
				{ PIXELP(lX, lY, 1, layer); }
		}	
		
	}
	
	//---------------------------------------------------------------------------------------------
	// Vertical (Y) division from top to bottom - 8 main divs and 5 sub divs
	//---------------------------------------------------------------------------------------------
	for(lY = y; lY < (y + GRID_HEIGHT + 1); lY++)
	{
		PIXELP(x - 1, lY, 1, layer);				// Left side line
		PIXELP(x + GRID_WIDTH + 1, lY, 1, layer);		// Right side line	
		// one point every second pixel
		if ((lY - y)%2 == 0) 
		{
			PIXELP(x + (GRID_WIDTH / 2), lY, 1, layer);	// Vertical middle line
		}	

		//draw subdivisions every 10th pixel - one square is 50 pixel high and has 5 subdivisions
		if ( ((lY - y)%10== 0) && ((lY - y) > 5) && ((lY - y) < (GRID_HEIGHT - 5)) && ((lY - y) != (GRID_HEIGHT / 2)) && ((lY - y)%50 != 0) )
		{
			//Left side 2 pixel line for 5 subdivisions
			PIXELP(x + 0, lY, 1, layer);
			PIXELP(x + 2, lY, 1, layer);
			//Middle left side 2 pixel line for 5 subdivisions
			PIXELP(x + (GRID_WIDTH / 2) - 3, lY, 1, layer);
			PIXELP(x + (GRID_WIDTH / 2) - 1, lY, 1, layer);
			//Middle right side 2 pixel line for 5 subdivisions
			PIXELP(x + (GRID_WIDTH / 2) + 1, lY, 1, layer);
			PIXELP(x + (GRID_WIDTH / 2) + 3, lY, 1, layer);
			//Right side 2 pixel line for 5 subdivisions
			PIXELP(x + GRID_WIDTH - 2, lY, 1, layer);
			PIXELP(x + GRID_WIDTH, lY, 1, layer);
		}		
		

		// Vertical main division 400 : 50 = 8 squares high
		if (((lY - y)%50 == 0) && ((lY - y) > 5) && ((lY - y) < (GRID_HEIGHT - 5)) && ((lY - y) != (GRID_HEIGHT / 2)))
		{
			// Left side 3 pixel line for 8 divisions a 50 pixel
			PIXELP(x + 1, lY, 1, layer);
			PIXELP(x + 3, lY, 1, layer);
			PIXELP(x + 5, lY, 1, layer);
			// Middle left side 3 pixel line for 8 divisions a 50 pixel
			PIXELP(x + (GRID_WIDTH / 2) - 6, lY, 1, layer);
			PIXELP(x + (GRID_WIDTH / 2) - 4, lY, 1, layer);
			PIXELP(x + (GRID_WIDTH / 2) - 2, lY, 1, layer);
			// Middle right side 3 pixel line for 8 divisions a 50 pixel
			PIXELP(x + (GRID_WIDTH / 2) + 2, lY, 1, layer);
			PIXELP(x + (GRID_WIDTH / 2) + 4, lY, 1, layer);
			PIXELP(x + (GRID_WIDTH / 2) + 6, lY, 1, layer);
			// Right side 3 pixel line for 8 divisions a 50 pixel
			PIXELP(x + GRID_WIDTH - 5, lY, 1, layer);
			PIXELP(x + GRID_WIDTH - 3, lY, 1, layer);
			PIXELP(x + GRID_WIDTH - 1, lY, 1, layer);

			//Gridlines horizontal - grid height modulo 50 = 8 lines with one point every 10th pixel
			if (dotted_grid)
				for (lX = x + 10 ; lX < (x + GRID_WIDTH - 5); lX += 10)	//dotted line
				{ PIXELP(lX, lY, 1, layer); }
			else
				for (lX = x; lX < (x + GRID_WIDTH); lX ++)	//solid line
				{ PIXELP(lX, lY, 1, layer); } 
		}		
	}
	
	//Gridlines horizontal - for 10% - 90% rise time determination with one point every 25th pixel
	lY = y + 75;		//90%
	//for (lX = x ; lX < (x + GRID_WIDTH); lX += 25)
	for (lX = x ; lX < (x + GRID_WIDTH); lX += 10)
	{	
		if (((lX - x) > 5) && ((lX - x) < (GRID_WIDTH - 5)))
		{
			//PIXELP(lX - 3, lY, 1, layer);
			PIXELP(lX, lY, 1, layer);
			//PIXELP(lX + 3, lY, 1, layer);
		}
	}

	lY = y + GRID_HEIGHT - 75;	//10%
	//for (lX = x ; lX < (x + GRID_WIDTH); lX += 25)
	for (lX = x ; lX < (x + GRID_WIDTH); lX += 10)
	{	
		if (((lX - x) > 5) && ((lX - x) < (GRID_WIDTH - 5)))
		{
			//PIXELP(lX - 3, lY, 1, layer);
			PIXELP(lX, lY, 1, layer);
			//PIXELP(lX + 3, lY, 1, layer);
		}
	}

	PIXELP(x - 1, y - 1, 1, layer); 				// Left top corner
	PIXELP(x + GRID_WIDTH + 1, y - 1, 1, layer); 			// Right top corner
	PIXELP(x - 1, y + GRID_HEIGHT + 1, 1, layer); 			// Left bottom corner
	PIXELP(x + GRID_WIDTH + 1, y + GRID_HEIGHT + 1, 1, layer); 	// Right bottom corner
}
//##########################################################################################################################################################

// BF changed (complete grid changed, new subdivisions and new grid dimension GRID_WIDTH 600 x 200 x 2)
void Display::GRIDDELAYED(unsigned long x, unsigned long y, unsigned long *layer) 		// Draw Grid subroutine
{
	unsigned long lX;
	unsigned long lY;
	unsigned long by = y + (GRID_HEIGHT / 2);	
	

	// Clear Gridplane
	for(lX =  GRID_PLANE_MIN; lX < GRID_PLANE_MAX; lX++)
	{ layer[lX] = 0; }	
	
	
	//Horizontal line drawing
	for(lX = x; lX < (x + GRID_WIDTH + 1); lX++)	//lX += 2)
	{
		PIXELP(lX, y - 1, 1, layer);										// Top Line
		PIXELP(lX, y + (GRID_HEIGHT / 4), 1, layer);								// Middle Line
		PIXELP(lX, y + ((GRID_HEIGHT / 2) + 1), 1, layer);							// Bottom Line
		
		
		if (((lX - x)%10 == 0) && ((lX - x) > 5) && ((lX - x) < (GRID_WIDTH - 5)) && ((lX - x) != (GRID_WIDTH / 2)) && ((lX - x)%50 != 0))
		{
			PIXELP(lX, y + 0, 1, layer); 									// 10 er Dots Top Line
			PIXELP(lX, y + 2, 1, layer);									// 10 er Dots Top Line
			
			PIXELP(lX, y + (GRID_HEIGHT / 4) - 3, 1, layer); 						// 10 er Dots Middle Line
			PIXELP(lX, y + (GRID_HEIGHT / 4) - 1, 1, layer); 						// 10 er Dots Middle Line
			PIXELP(lX, y + (GRID_HEIGHT / 4) + 1, 1, layer); 						// 10 er Dots Middle Line
			PIXELP(lX, y + (GRID_HEIGHT / 4) + 3, 1, layer); 						// 10 er Dots Middle Line
			
			PIXELP(lX, y + (GRID_HEIGHT / 2) - 2, 1, layer);						// 10 er Dots Bottom Line
			PIXELP(lX, y + (GRID_HEIGHT / 2), 1, layer); 							// 10 er Dots Bottom Line
	
		}		

		if (((lX - x)%50 == 0) && ((lX - x) > 5) && ((lX - x) < (GRID_WIDTH - 5)) && ((lX - x) != (GRID_WIDTH / 2)))
		{

			PIXELP(lX, y + 1, 1, layer); 									// 10 er Dots Top Line
			PIXELP(lX, y + 3, 1, layer); 									// 10 er Dots Top Line	
			PIXELP(lX, y + 5, 1, layer); 									// 10 er Dots Top Line	
			
			PIXELP(lX, y + (GRID_HEIGHT / 4) - 6, 1, layer); 						// 10 er Dots Middle Line	
			PIXELP(lX, y + (GRID_HEIGHT / 4) - 4, 1, layer); 						// 10 er Dots Middle Line
			PIXELP(lX, y + (GRID_HEIGHT / 4) - 2, 1, layer);						// 10 er Dots Middle Line
			PIXELP(lX, y + (GRID_HEIGHT / 4) + 2, 1, layer); 						// 10 er Dots Middle Line
			PIXELP(lX, y + (GRID_HEIGHT / 4) + 4, 1, layer); 						// 10 er Dots Middle Line
			PIXELP(lX, y + (GRID_HEIGHT / 4) + 6, 1, layer); 						// 10 er Dots Middle Line
			
			PIXELP(lX, y + (GRID_HEIGHT / 2) - 5, 1, layer); 						// 10 er Dots Bottom Line
			PIXELP(lX, y + (GRID_HEIGHT / 2) - 3, 1, layer); 						// 10 er Dots Bottom Line
			PIXELP(lX, y + (GRID_HEIGHT / 2) - 1, 1, layer); 						// 10 er Dots Bottom Line	

			//Draw vertical line on every main X-division
			for(lY = (y + 5); lY < (y + (GRID_HEIGHT / 2)); lY += 5)	// 10 BF ???
			{
				PIXELP(lX, lY, 1, layer); 								// 50 er Dots
			}
		}
		
	}
	
	//Vertical line drawing
	for(lY = y; lY < (y + ((GRID_HEIGHT / 2) + 1)); lY++)	//lY += 2) 
	{
		PIXELP(x - 1, lY, 1, layer);										// Left Line
		PIXELP(x + (GRID_WIDTH / 2), lY, 1, layer);								// Middle Line
		PIXELP(x + GRID_WIDTH + 1, lY, 1, layer);								// Right Line
		
		if (((lY - y)%5 == 0) && ((lY - y) > 2) && ((lY - y) < ((GRID_HEIGHT / 2) - 2)) && ((lY - y) != (GRID_HEIGHT / 4)) && ((lY - y)%25 != 0))
		{
			PIXELP(x + 0, lY, 1, layer); 									// 10 er Dots Left Line
			PIXELP(x + 2, lY, 1, layer); 									// 10 er Dots Left Line
			PIXELP(x + (GRID_WIDTH / 2) - 3, lY, 1, layer); 						// 10 er Dots Middle Line
			PIXELP(x + (GRID_WIDTH / 2) - 1, lY, 1, layer); 						// 10 er Dots Middle Line
			PIXELP(x + (GRID_WIDTH / 2) + 1, lY, 1, layer); 						// 10 er Dots Middle Line
			PIXELP(x + (GRID_WIDTH / 2) + 3, lY, 1, layer); 						// 10 er Dots Middle Line
			PIXELP(x + GRID_WIDTH - 2, lY, 1, layer); 							// 10 er Dots Right Line
			PIXELP(x + GRID_WIDTH, lY, 1, layer); 								// 10 er Dots Right Line
		}		
		
		if (((lY - y)%25 == 0) && ((lY - y) > 2) && ((lY - y) < ((GRID_HEIGHT / 2) - 2)) && ((lY - y) != (GRID_HEIGHT / 4)))
		{

			PIXELP(x + 1, lY, 1, layer); 									// 10 er Dots Left Line
			PIXELP(x + 3, lY, 1, layer); 									// 10 er Dots Left Line
			PIXELP(x + 5, lY, 1, layer); 									// 10 er Dots Left Line
			
			PIXELP(x + (GRID_WIDTH / 2) - 6, lY, 1, layer); 						// 10 er Dots Middle Line	
			PIXELP(x + (GRID_WIDTH / 2) - 4, lY, 1, layer); 						// 10 er Dots Middle Line
			PIXELP(x + (GRID_WIDTH / 2) - 2, lY, 1, layer); 						// 10 er Dots Middle Line
			PIXELP(x + (GRID_WIDTH / 2) + 2, lY, 1, layer); 						// 10 er Dots Middle Line
			PIXELP(x + (GRID_WIDTH / 2) + 4, lY, 1, layer); 						// 10 er Dots Middle Line
			PIXELP(x + (GRID_WIDTH / 2) + 6, lY, 1, layer); 						// 10 er Dots Middle Line
			
			PIXELP(x + GRID_WIDTH - 5, lY, 1, layer); 							// 10 er Dots Right Line
			PIXELP(x + GRID_WIDTH - 3, lY, 1, layer); 							// 10 er Dots Right Line	
			PIXELP(x + GRID_WIDTH - 1, lY, 1, layer); 							// 10 er Dots Right Line

			//Draw horizontal line on every main Y-division
			for (lX = (x + 10); lX < (x + GRID_WIDTH); lX += 10)
			{
				PIXELP(lX, lY, 1, layer); 								// 50 er Dots
			}
		}		
	}
	
	PIXELP(x - 1, y - 1, 1, layer); 										// Left Top corner
	PIXELP(x + GRID_WIDTH + 1, y - 1, 1, layer);									// Right Top corner
	PIXELP(x - 1, y + ((GRID_HEIGHT / 2) + 1), 1, layer); 								// Left Bottom corner
	PIXELP(x + GRID_WIDTH + 1, y + ((GRID_HEIGHT / 2) + 1), 1, layer); 						// Right Bottom corner

	//-----------------------------------	
	// 	Delayed Part
	//-----------------------------------
	// Horizontal drawing
	for (lX = x; lX < (x + GRID_WIDTH + 1);	lX++)	//lX += 2)
	{
		PIXELP(lX, by - 1, 1, layer);										// Top Line
		PIXELP(lX, by + (GRID_HEIGHT / 4), 1, layer);								// Middle Line
		PIXELP(lX, by + ((GRID_HEIGHT / 2) + 1), 1, layer);							// Bottom Line
		
		
		if (((lX - x)%10 == 0) && ((lX - x) > 5) && ((lX - x) < (GRID_WIDTH - 5)) && ((lX - x) != (GRID_WIDTH / 2)) && ((lX - x)%50 != 0))
		{
			PIXELP(lX, by + 0, 1, layer); 									// 10 er Dots Top Line
			PIXELP(lX, by + 2, 1, layer); 									// 10 er Dots Top Line
			PIXELP(lX, by + (GRID_HEIGHT / 4) - 3, 1, layer); 						// 10 er Dots Middle Line
			PIXELP(lX, by + (GRID_HEIGHT / 4) - 1, 1, layer); 						// 10 er Dots Middle Line
			PIXELP(lX, by + (GRID_HEIGHT / 4) + 1, 1, layer); 						// 10 er Dots Middle Line
			PIXELP(lX, by + (GRID_HEIGHT / 4) + 3, 1, layer); 						// 10 er Dots Middle Line
			PIXELP(lX, by + (GRID_HEIGHT / 2) - 2, 1, layer);						// 10 er Dots Bottom Line
			PIXELP(lX, by + (GRID_HEIGHT / 2), 1, layer); 							// 10 er Dots Bottom Line
	
		}		

		if (((lX - x)%50 == 0) && ((lX - x) > 5) && ((lX - x) < (GRID_WIDTH - 5)) && ((lX - x) != (GRID_WIDTH / 2)))
		{

			PIXELP(lX, by + 1, 1, layer); 									// 10 er Dots Top Line
			PIXELP(lX, by + 3, 1, layer); 									// 10 er Dots Top Line
			PIXELP(lX, by + 5, 1, layer); 									// 10 er Dots Top Line
			
			PIXELP(lX, by + (GRID_HEIGHT / 4) - 6, 1, layer); 						// 10 er Dots Middle Line
			PIXELP(lX, by + (GRID_HEIGHT / 4) - 4, 1, layer); 						// 10 er Dots Middle Line
			PIXELP(lX, by + (GRID_HEIGHT / 4) - 2, 1, layer); 						// 10 er Dots Middle Line
			PIXELP(lX, by + (GRID_HEIGHT / 4) + 2, 1, layer); 						// 10 er Dots Middle Line
			PIXELP(lX, by + (GRID_HEIGHT / 4) + 4, 1, layer); 						// 10 er Dots Middle Line
			PIXELP(lX, by + (GRID_HEIGHT / 4) + 6, 1, layer); 						// 10 er Dots Middle Line
			
			PIXELP(lX, by + (GRID_HEIGHT / 2) - 5, 1, layer); 						// 10 er Dots Bottom Line
			PIXELP(lX, by + (GRID_HEIGHT / 2) - 3, 1, layer); 						// 10 er Dots Bottom Line
			PIXELP(lX, by + (GRID_HEIGHT / 2) - 1, 1, layer);						// 10 er Dots Bottom Line	

			//Draw vertical line on every main X-division
			for (lY = (by + 5); lY < (by + (GRID_HEIGHT / 2)); lY += 5)	//10  BF ??
			{
				PIXELP(lX, lY, 1, layer); 								// 50 er Dots
			}
		}
		
	}
	
	//Vertical drawing
	for	(lY = by; lY < (by + ((GRID_HEIGHT / 2) + 1)); lY++) 	//lY += 2)
	{
		PIXELP(x - 1, lY, 1, layer);										// 2 er Dots Left Line
		PIXELP(x + (GRID_WIDTH / 2), lY, 1, layer);								// 2 er Dots Middle Line
		PIXELP(x + GRID_WIDTH + 1, lY, 1, layer);								// 2 er Dots Right Line		
		
		if (((lY - by)%5 == 0) && ((lY - by) > 2) && ((lY - by) < ((GRID_HEIGHT / 2) - 2)) && ((lY - by) != (GRID_HEIGHT / 4)) && ((lY - by)%25 != 0))
		{
			PIXELP(x + 0, lY, 1, layer); 									// 10 er Dots Left Line
			PIXELP(x + 2, lY, 1, layer); 									// 10 er Dots Left Line
			PIXELP(x + (GRID_WIDTH / 2) - 3, lY, 1, layer); 						// 10 er Dots Middle Line
			PIXELP(x + (GRID_WIDTH / 2) - 1, lY, 1, layer); 						// 10 er Dots Middle Line
			PIXELP(x + (GRID_WIDTH / 2) + 1, lY, 1, layer);							// 10 er Dots Middle Line
			PIXELP(x + (GRID_WIDTH / 2) + 3, lY, 1, layer); 						// 10 er Dots Middle Line
			PIXELP(x + GRID_WIDTH - 2, lY, 1, layer); 							// 10 er Dots Right Line
			PIXELP(x + GRID_WIDTH, lY, 1, layer); 								// 10 er Dots Right Line
		}		
		
		if (((lY - by)%25 == 0) && ((lY - by) > 2) && ((lY - by) < ((GRID_HEIGHT / 2) - 2)) && ((lY - by) != (GRID_HEIGHT / 4)))
		{

			PIXELP(x + 1, lY, 1, layer); 									// 10 er Dots Left Line
			PIXELP(x + 3, lY, 1, layer); 									// 10 er Dots Left Line
			PIXELP(x + 5, lY, 1, layer); 									// 10 er Dots Left Line
			PIXELP(x + (GRID_WIDTH / 2) - 6, lY, 1, layer); 						// 10 er Dots Middle Line
			PIXELP(x + (GRID_WIDTH / 2) - 4, lY, 1, layer); 						// 10 er Dots Middle Line
			PIXELP(x + (GRID_WIDTH / 2) - 2, lY, 1, layer); 						// 10 er Dots Middle Line
			PIXELP(x + (GRID_WIDTH / 2) + 2, lY, 1, layer); 						// 10 er Dots Middle Line
			PIXELP(x + (GRID_WIDTH / 2) + 4, lY, 1, layer); 						// 10 er Dots Middle Line
			PIXELP(x + (GRID_WIDTH / 2) + 6, lY, 1, layer); 						// 10 er Dots Middle Line
			PIXELP(x + GRID_WIDTH - 5, lY, 1, layer); 							// 10 er Dots Right Line
			PIXELP(x + GRID_WIDTH - 3, lY, 1, layer); 							// 10 er Dots Right Line
			PIXELP(x + GRID_WIDTH - 1, lY, 1, layer); 							// 10 er Dots Right Line

			//Draw horizontal line on every main Y-division
			for (lX = (x + 10); lX < (x + GRID_WIDTH); lX += 10)
			{
				PIXELP(lX, lY, 1, layer); 								// 50 er Dots
			}
		}		
	}
	
	PIXELP(x - 1, by - 1, 1, layer); 										// Left Top corner
	PIXELP(x + GRID_WIDTH + 1, by - 1, 1, layer); 									// Right Top corner
	PIXELP(x - 1, by + ((GRID_HEIGHT / 2) + 1), 1, layer); 								// Left Bottom corner
	PIXELP(x + GRID_WIDTH + 1, by + ((GRID_HEIGHT / 2) + 1), 1, layer); 						// Right Bottom corner
/*
	for (lX = x; lX < (x + GRID_WIDTH + 1); lX ++)
	{
		PIXELP(lX, by - 1, 1, layer);										// Top Line
		PIXELP(lX, by + 0, 1, layer);										// Middle Line
		PIXELP(lX, by + 1, 1, layer);										// Bottom Line
    	}
*/
}	//Grid delayed end
//##########################################################################################################################################################

// BF new grid for FFT function with dimension 512 x 400
void Display::GRIDFFT(unsigned long x, unsigned long y, unsigned long *layer) 		// Draw Grid subroutine
{
	unsigned long lX;
	unsigned long lY;
	
	// Clear Gridplane
	for(lX =  GRID_PLANE_MIN; lX < GRID_PLANE_MAX; lX++)
	{ layer[lX] = 0; }	

	// grid for linear division
	if(FFT_Mode == FFT_MAGN || FFT_Mode == FFT_REAL || FFT_Mode == FFT_IMAG || FFT_Mode == FFT_REIM || FFT_Mode == FFT_PHAS)
	{
		//---------------------------------------------------------------------------------------------
		// Horizontal lines (X) division from left to right - 12 main divs and 5 sub divs
		//---------------------------------------------------------------------------------------------
		for (lX = x; lX < (x + GRID_WIDTH_FFT + 1); lX ++)
		{
			//draw top and bottom edge lines and middle line, 
			PIXELP(lX, y-1, 1, layer);				// Top Line
			PIXELP(lX, y + GRID_HEIGHT_FFT + 1, 1, layer);		// Bottom Line
			
			
			//draw subdivisions every 10th pixel - one square is 50 pixel wide and has 5 subdivisions
			if( ((lX - x)%16 == 0)&&((lX - x) > 5) && ((lX - x) < (GRID_WIDTH_FFT - 5)) && ((lX - x) != (GRID_WIDTH_FFT / 2)) && ((lX - x)%64 != 0) )
			{
				//Top 2 pixel line for 5 subdivisions (every 10th x-pixel)
				PIXELP(lX, y + 0, 1, layer);
				PIXELP(lX, y + 2, 1, layer);
			}	
	
			//Main division - 64 pixel for one division
			if (((lX - x)%64 == 0) && ((lX - x) > 5) && ((lX - x) < GRID_WIDTH_FFT - 5))
			{
				//Top line division
				PIXELP(lX, y + 1, 1, layer);
				PIXELP(lX, y + 3, 1, layer);
				PIXELP(lX, y + 5, 1, layer);
	
				//Grid lines vertical - grid width modulo 64 = 8 lines 
				if (dotted_grid)
					for (lY = y + 10; lY < (y + GRID_HEIGHT_FFT - 5); lY += 10)	//dotted line with one point every 10th Y-pixel
					{ PIXELP(lX, lY, 1, layer); }
				else
					for (lY = y; lY < (y + GRID_HEIGHT_FFT); lY ++)			//solid line
					{ PIXELP(lX, lY, 1, layer); }
			}	
			
		}
		
		//---------------------------------------------------------------------------------------------
		// Vertical lines (Y) division from top to bottom - 8 main divs and 5 sub divs
		//---------------------------------------------------------------------------------------------
		for(lY = y; lY < (y + GRID_HEIGHT_FFT + 1); lY++)
		{
			PIXELP(x-1, lY, 1, layer);				// Left side line
			PIXELP(x + GRID_WIDTH_FFT + 1, lY, 1, layer);		// Right side line	
	
			//draw subdivisions every 10th pixel - one square is 50 pixel high and has 5 subdivisions
			if ( ((lY - y)%10== 0) && ((lY - y) > 5) && ((lY - y) < (GRID_HEIGHT - 5)) && ((lY - y) != (GRID_HEIGHT / 2)) && ((lY - y)%50 != 0) )
			{
				//Left side 2 pixel line for 5 subdivisions
				PIXELP(x + 0, lY, 1, layer);
				PIXELP(x + 2, lY, 1, layer);
	
				//Right side 2 pixel line for 5 subdivisions
				PIXELP(x + GRID_WIDTH_FFT - 2, lY, 1, layer);
				PIXELP(x + GRID_WIDTH_FFT, lY, 1, layer);
			}		
	
	
			// Vertical main division 400 : 50 = 8 squares high
			if (((lY - y)%50 == 0) && ((lY - y) > 5) && ((lY - y) < (GRID_HEIGHT_FFT - 5)) )
			{
				// Left side 3 pixel line for 8 divisions a 50 pixel
				PIXELP(x + 1, lY, 1, layer);
				PIXELP(x + 3, lY, 1, layer);
				PIXELP(x + 5, lY, 1, layer);
	
				// Right side 3 pixel line for 8 divisions a 50 pixel
				PIXELP(x + GRID_WIDTH_FFT - 5, lY, 1, layer);
				PIXELP(x + GRID_WIDTH_FFT - 3, lY, 1, layer);
				PIXELP(x + GRID_WIDTH_FFT - 1, lY, 1, layer);
	
				//Gridlines horizontal - grid height modulo 50 = 8 lines 
				if (dotted_grid)
					for (lX = x + 16 ; lX < (x + GRID_WIDTH_FFT - 5); lX += 16)	//dotted line with one point every 16th X-pixel
					{ PIXELP(lX, lY, 1, layer); }
				else
					for (lX = x; lX < (x + GRID_WIDTH_FFT); lX ++)			//solid line
					{ PIXELP(lX, lY, 1, layer); } 
			}		
		}
	}
	else if (FFT_Mode == FFT_PWSP || FFT_Mode == FFT_PWDN)	// grid for logarithmic division
	{
		//---------------------------------------------------------------------------------------------
		// Horizontal lines (X) division from left to right - 12 main divs and 5 sub divs
		//---------------------------------------------------------------------------------------------
		for (lX = x; lX < (x + GRID_WIDTH_FFT + 1); lX ++)
		{
			//draw top and bottom edge lines and middle line, 
			PIXELP(lX, y-1, 1, layer);				// Top Line
			PIXELP(lX, y + GRID_HEIGHT_FFT + 1, 1, layer);		// Bottom Line
			
			
			//draw subdivisions every 10th pixel - one square is 50 pixel wide and has 5 subdivisions
			if( ((lX - x)%16 == 0)&&((lX - x) > 5) && ((lX - x) < (GRID_WIDTH_FFT - 5)) && ((lX - x) != (GRID_WIDTH_FFT / 2)) && ((lX - x)%64 != 0) )
			{
				//Top 2 pixel line for 5 subdivisions (every 10th x-pixel)
				PIXELP(lX, y + 0, 1, layer);
				PIXELP(lX, y + 2, 1, layer);
			}	
	
			//Main division - 64 pixel for one division
			if (((lX - x)%64 == 0) && ((lX - x) > 5) && ((lX - x) < GRID_WIDTH_FFT - 5))
			{
				//Top line division
				PIXELP(lX, y + 1, 1, layer);
				PIXELP(lX, y + 3, 1, layer);
				PIXELP(lX, y + 5, 1, layer);
	
				//Grid lines vertical - grid width modulo 64 = 8 lines 
				if (dotted_grid)
					for (lY = y + 10; lY < (y + GRID_HEIGHT_FFT - 5); lY += 10)	//dotted line with one point every 10th Y-pixel
					{ PIXELP(lX, lY, 1, layer); }
				else
					for (lY = y; lY < (y + GRID_HEIGHT_FFT); lY ++)			//solid line
					{ PIXELP(lX, lY, 1, layer); }
			}	
			
		}
		
		//---------------------------------------------------------------------------------------------
		// Vertical lines (Y) division from top to bottom - 8 main divs and 5 sub divs
		//---------------------------------------------------------------------------------------------
		for(lY = y; lY < (y + GRID_HEIGHT_FFT + 1); lY++)
		{
			PIXELP(x-1, lY, 1, layer);				// Left side line
			PIXELP(x + GRID_WIDTH_FFT + 1, lY, 1, layer);		// Right side line	
	
			//draw subdivisions every 10th pixel - one square is 50 pixel high and has 5 subdivisions
			//if ( ((lY - y)%10== 0) && ((lY - y) > 5) && ((lY - y) < (GRID_HEIGHT - 5) ) && ((lY - y) != (GRID_HEIGHT / 2)) && ((lY - y)%50 != 0) )
			if ( ((lY - y)%8== 0) && ((lY - y) > 5) && ((lY - y) < (GRID_HEIGHT - 5) ) && ((lY - y)%80 != 0) )
			{
				//Left side 2 pixel line for 10 subdivisions
				PIXELP(x + 0, lY, 1, layer);
				PIXELP(x + 2, lY, 1, layer);
	
				//Right side 2 pixel line for 10 subdivisions
				PIXELP(x + GRID_WIDTH_FFT - 2, lY, 1, layer);
				PIXELP(x + GRID_WIDTH_FFT, lY, 1, layer);
			}		
	
	
			// Vertical main division 400 : 80 = 8 squares high
			if (((lY - y)%80 == 0) && ((lY - y) > 5) && ((lY - y) < (GRID_HEIGHT_FFT - 5)) )
			{
				// Left side 3 pixel line for 8 divisions a 50 pixel
				PIXELP(x + 1, lY, 1, layer);
				PIXELP(x + 3, lY, 1, layer);
				PIXELP(x + 5, lY, 1, layer);
	
				// Right side 3 pixel line for 8 divisions a 50 pixel
				PIXELP(x + GRID_WIDTH_FFT - 5, lY, 1, layer);
				PIXELP(x + GRID_WIDTH_FFT - 3, lY, 1, layer);
				PIXELP(x + GRID_WIDTH_FFT - 1, lY, 1, layer);
	
				//Gridlines horizontal - grid height modulo 80 = 5 lines 
				if (dotted_grid)
					for (lX = x + 16 ; lX < (x + GRID_WIDTH_FFT - 5); lX += 16)	//dotted line with one point every 16th X-pixel
					{ PIXELP(lX, lY, 1, layer); }
				else
					for (lX = x; lX < (x + GRID_WIDTH_FFT); lX ++)			//solid line
					{ PIXELP(lX, lY, 1, layer); } 
			}		
		}
	}

	PIXELP(x - 1, y - 1, 1, layer); 					// Left top corner
	PIXELP(x + GRID_WIDTH_FFT + 1, y - 1, 1, layer); 			// Right top corner
	PIXELP(x - 1, y + GRID_HEIGHT_FFT + 1, 1, layer); 			// Left bottom corner
	PIXELP(x + GRID_WIDTH_FFT + 1, y + GRID_HEIGHT_FFT + 1, 1, layer); 	// Right bottom corner


	
}
//##########################################################################################################################################################

// BF changed (complete grid changed, new subdivisions and new grid dimension 400 x 400)
void Display::GRIDXY(unsigned long x, unsigned long y, unsigned long *layer) 		// Draw Grid subroutine
{
	unsigned long lX;
	unsigned long lY;

	// Clear Gridplane
	for(lX =  GRID_PLANE_MIN; lX < GRID_PLANE_MAX; lX++)
	{ layer[lX] = 0; }	


	//---------------------------------------------------------------------------------------------
	// Horizontal (X) division from left to right - 8 main divs and 5 sub divs
	//---------------------------------------------------------------------------------------------
	for (lX = x; lX < (x + GRID_XY + 1); lX ++)
	{
		//draw top and bottom edge lines and middle line, 
		PIXELP(lX, y - 1, 1, layer);			// Top Line
		PIXELP(lX, y + GRID_XY + 1, 1, layer);		// Bottom Line
		//print every second point 
		if ((lX - x)%2 == 0) {
			PIXELP(lX, y + GRID_XY / 2, 1, layer);	// Middle Line
		}	
		
		
		//draw subdivisions every 10th pixel - one square is 50 pixel wide and has 5 subdivisions
		if( ((lX - x)%10 == 0)&&((lX - x) > 5) && ((lX - x) < (GRID_XY - 5)) && ((lX - x) != (GRID_XY / 2)) && ((lX - x)%50 != 0) )
		{
			//Top 2 pixel line for 5 subdivisions (every 10th x-pixel)
			PIXELP(lX, y + 0, 1, layer);
			PIXELP(lX, y + 2, 1, layer);
			//Middle top 2 pixel line for 5 subdivisions
			PIXELP(lX, y + (GRID_XY / 2) - 3, 1, layer);
			PIXELP(lX, y + (GRID_XY / 2) - 1, 1, layer);
			//Middle bottom 2 pixel line for 5 subdivisions
			PIXELP(lX, y + (GRID_XY / 2) + 1, 1, layer);
			PIXELP(lX, y + (GRID_XY / 2) + 3, 1, layer);
			//Bottom 2 pixel line for 5 subdivisions
			PIXELP(lX, y + GRID_XY - 2, 1, layer);
			PIXELP(lX, y + GRID_XY, 1, layer);
		}	

		//Main division - 50 pixel for one divisin
		if (((lX - x)%50 == 0) && ((lX - x) > 5) && ((lX - x) < GRID_XY - 5) && ((lX - x) != (GRID_XY / 2)))
		{
			//Top line division
			PIXELP(lX, y + 1, 1, layer);
			PIXELP(lX, y + 3, 1, layer);
			PIXELP(lX, y + 5, 1, layer);
			//Middle line top division
			PIXELP(lX, y + ((GRID_XY / 2) - 6), 1, layer);
			PIXELP(lX, y + ((GRID_XY / 2) - 4), 1, layer);
			PIXELP(lX, y + ((GRID_XY / 2) - 2), 1, layer);
			//Middle line bottom division
			PIXELP(lX, y + ((GRID_XY / 2) + 2), 1, layer);
			PIXELP(lX, y + ((GRID_XY / 2) + 4), 1, layer);
			PIXELP(lX, y + ((GRID_XY / 2) + 6), 1, layer);
			//Bottom line division
			PIXELP(lX, y + (GRID_XY - 5), 1, layer);
			PIXELP(lX, y + (GRID_XY - 3), 1, layer);
			PIXELP(lX, y + (GRID_XY - 1), 1, layer);

			//Grid lines vertical - grid width modulo 50 = 8 lines with one point every 10th pixel
			if (dotted_grid)
			for (lY = y + 10; lY < (y + GRID_XY - 5); lY += 10)	//dotted line
			{ PIXELP(lX, lY, 1, layer); }
			else
			for (lY = y; lY < (y + GRID_XY); lY ++)		//solid line
			{ PIXELP(lX, lY, 1, layer); }
		}	
		
	}
	
	//---------------------------------------------------------------------------------------------
	// Vertical (Y) division from top to bottom - 8 main divs and 5 sub divs
	//---------------------------------------------------------------------------------------------
	for(lY = y; lY < (y + GRID_XY + 1); lY++)
	{
		PIXELP(x - 1, lY, 1, layer);				// Left side line
		PIXELP(x + GRID_XY + 1, lY, 1, layer);			// Right side line	
		// one point every second pixel
		if ((lY - y)%2 == 0) 
		{
			PIXELP(x + (GRID_XY / 2), lY, 1, layer);	// Vertical middle line
		}	

		//draw subdivisions every 10th pixel - one square is 50 pixel high and has 5 subdivisions
		if ( ((lY - y)%10== 0) && ((lY - y) > 5) && ((lY - y) < (GRID_XY - 5)) && ((lY - y) != (GRID_XY / 2)) && ((lY - y)%50 != 0) )
		{
			//Left side 2 pixel line for 5 subdivisions
			PIXELP(x + 0, lY, 1, layer);
			PIXELP(x + 2, lY, 1, layer);
			//Middle left side 2 pixel line for 5 subdivisions
			PIXELP(x + (GRID_XY / 2) - 3, lY, 1, layer);
			PIXELP(x + (GRID_XY / 2) - 1, lY, 1, layer);
			//Middle right side 2 pixel line for 5 subdivisions
			PIXELP(x + (GRID_XY / 2) + 1, lY, 1, layer);
			PIXELP(x + (GRID_XY / 2) + 3, lY, 1, layer);
			//Right side 2 pixel line for 5 subdivisions
			PIXELP(x + GRID_XY - 2, lY, 1, layer);
			PIXELP(x + GRID_XY, lY, 1, layer);
		}		
		

		// Vertical main division 400 : 50 = 8 squares high
		if (((lY - y)%50 == 0) && ((lY - y) > 5) && ((lY - y) < (GRID_XY - 5)) && ((lY - y) != (GRID_XY / 2)))
		{
			// Left side 3 pixel line for 8 divisions a 50 pixel
			PIXELP(x + 1, lY, 1, layer);
			PIXELP(x + 3, lY, 1, layer);
			PIXELP(x + 5, lY, 1, layer);
			// Middle left side 3 pixel line for 8 divisions a 50 pixel
			PIXELP(x + (GRID_XY / 2) - 6, lY, 1, layer);
			PIXELP(x + (GRID_XY / 2) - 4, lY, 1, layer);
			PIXELP(x + (GRID_XY / 2) - 2, lY, 1, layer);
			// Middle right side 3 pixel line for 8 divisions a 50 pixel
			PIXELP(x + (GRID_XY / 2) + 2, lY, 1, layer);
			PIXELP(x + (GRID_XY / 2) + 4, lY, 1, layer);
			PIXELP(x + (GRID_XY / 2) + 6, lY, 1, layer);
			// Right side 3 pixel line for 8 divisions a 50 pixel
			PIXELP(x + GRID_XY - 5, lY, 1, layer);
			PIXELP(x + GRID_XY - 3, lY, 1, layer);
			PIXELP(x + GRID_XY - 1, lY, 1, layer);

			//Gridlines horizontal - grid height modulo 50 = 8 lines with one point every 10th pixel
			if (dotted_grid)
			for (lX = x + 10 ; lX < (x + GRID_XY - 5); lX += 10)	//dotted line
			{ PIXELP(lX, lY, 1, layer); }
			else
			for (lX = x; lX < (x + GRID_XY); lX ++)		//solid line
			{ PIXELP(lX, lY, 1, layer); }
		}		
	}

	//Corners
	PIXELP(x - 1, y - 1, 1, layer); 			// Left top corner
	PIXELP(x + GRID_XY + 1, y - 1, 1, layer); 		// Right top corner
	PIXELP(x - 1, y + GRID_XY + 1, 1, layer); 		// Left bottom corner
	PIXELP(x + GRID_XY + 1, y + GRID_XY + 1, 1, layer); 	// Right bottom corner
}
// Public Functions
void Display::Init(void)											// Init Display Subroutine
{

	unsigned long *flash_address;
	int x, result = 0;
#ifdef _Debug_	
	if (Debug_Mode) printf("Starting Init Display \n");
#endif			
	Pulse11.Init(0.000000016, 0.000000016, 0.268435448, 0.000000008, 2, 3, "", "", "s");
	Pulse12.Init(0.000000008, 0.000000008, 0.000524000, 0.000000008, 2, 3, "", "", "s");
	Pulse21.Init(0.000000016, 0.000000016, 0.268435448, 0.000000008, 2, 3, "", "", "s");
	Pulse22.Init(0.000000016, 0.000000016, 0.268435448, 0.000000008, 2, 3, "", "", "s");
	
        switch(Pulse11_Expo)
        {
            case 1:
            {
                Pulse11.Write_Value((float) Pulse11_Value / (float) 1000000000);
                break;
            }
            case 2:
            {
                Pulse11.Write_Value((float) Pulse11_Value / (float) 1000000);
                break;
            }
            case 3:
            {
                Pulse11.Write_Value((float) Pulse11_Value / (float) 1000);
                break;
            }
        }
        Pulse11.RenderText();

        switch(Pulse12_Expo)
        {
            case 1:
            {
                Pulse12.Write_Value((float) Pulse12_Value / (float) 1000000000);
                break;
            }
            case 2:
            {
                Pulse12.Write_Value((float) Pulse12_Value / (float) 1000000);
                break;
            }
            case 3:
            {
                Pulse12.Write_Value((float) Pulse12_Value / (float) 1000);
                break;
            }
        }
        Pulse12.RenderText();

        switch(Pulse21_Expo)
        {
            case 1:
            {
                Pulse21.Write_Value((float) Pulse21_Value / (float) 1000000000);
                break;
            }
            case 2:
            {
                Pulse21.Write_Value((float) Pulse21_Value / (float) 1000000);
                break;
            }
            case 3:
            {
                Pulse21.Write_Value((float) Pulse21_Value / (float) 1000);
                break;
            }
        }
        Pulse21.RenderText();

        switch(Pulse22_Expo)
        {
            case 1:
            {
                Pulse22.Write_Value((float) Pulse22_Value / (float) 1000000000);
                break;
            }
            case 2:
            {
                Pulse22.Write_Value((float) Pulse22_Value / (float) 1000000);
                break;
            }
            case 3:
            {
                Pulse22.Write_Value((float) Pulse22_Value / (float) 1000);
                break;
            }
        }
        Pulse22.RenderText();	

	// init normal cursor data display
	Cursor_Delta_X.Init(0.00000001, 0.00000001, 10, 0.000000005, 2, 4, "", "@X = ", "s");
	Cursor_Delta_1X.Init(0.00000001, 0.00000001, 10, 0.000000005, 3, 5, "", "1/@X = ", "Hz");
	Cursor_Delta_XY_X.Init(0.00000001, 0.00000001, 10, 0.000000005, 2, 4, "", "", "V");
	Cursor_Delta_Y.Init(0.00000001, 0.00000001, 10, 0.000000005, 2, 4, "", "", "V");

	Cursor_Vertical_Pos_1_Str.Init(0.00000001, 0.00000001, 10, 0.000000005, 2, 4, "", "", "s");
	Cursor_Vertical_Pos_2_Str.Init(0.00000001, 0.00000001, 10, 0.000000005, 2, 4, "", "", "s");

	Cursor_Vertical_Pos_XY_1_Str.Init(0.00000001, 0.00000001, 10, 0.000000005, 2, 4, "", "", "V");
	Cursor_Vertical_Pos_XY_2_Str.Init(0.00000001, 0.00000001, 10, 0.000000005, 2, 4, "", "", "V");

	Cursor_Horizontal_Pos_1_Str.Init(0.00000001, 0.00000001, 10, 0.000000005, 2, 4, "", "", "V");
	Cursor_Horizontal_Pos_2_Str.Init(0.00000001, 0.00000001, 10, 0.000000005, 2, 4, "", "", "V");

	// init FFT cursor data display
	FFT_Cursor_Delta_X.Init(0.00000001, 0.00000001, 10, 0.000000005, 2, 3, "", "@X = ", "Hz");
	FFT_Cursor_Delta_1X.Init(0.00000001, 0.00000001, 10, 0.000000005, 3, 5, "", "1/@X = ", "s");
	FFT_Cursor_Delta_Y.Init(0.00000001, 0.00000001, 10, 0.000000005, 2, 3, "", "", "V");

	FFT_Cursor_Vertical_Pos_1_Str.Init(0.00000001, 0.00000001, 10, 0.000000005, 2, 3, "", "", "Hz");
	FFT_Cursor_Vertical_Pos_2_Str.Init(0.00000001, 0.00000001, 10, 0.000000005, 2, 3, "", "", "Hz");

	FFT_Cursor_Horizontal_Pos_1_Str.Init(0.00000001, 0.00000001, 10, 0.000000005, 2, 3, "", "", "V");
	FFT_Cursor_Horizontal_Pos_2_Str.Init(0.00000001, 0.00000001, 10, 0.000000005, 2, 3, "", "", "V");

	// init QM-data display
	QM_Slot1.Init(0.00000001, 0.00000001, 10, 0.000000005, 2, 3, "", "Freq(@) :", "");
	QM_Slot2.Init(0.00000001, 0.00000001, 10, 0.000000005, 2, 3, "", "Pk-Pk(@) :", "");
	QM_Slot3.Init(0.00000001, 0.00000001, 10, 0.000000005, 2, 3, "", "Pk-Pk(@) :", "");	

//	QM_Slot1_Value.Init(0.00000001, 0.00000001, 10, 0.000000005, 2, 3, "", "", "Hz");
//	QM_Slot2_Value.Init(0.00000001, 0.00000001, 10, 0.000000005, 2, 3, "", "", "V");
//	QM_Slot3_Value.Init(0.00000001, 0.00000001, 10, 0.000000005, 2, 3, "", "", "V");	

	QM_Threshold_lower.Init(30, 5, 100, 1, 0, 3, "", "", "%");
	QM_Threshold_middle.Init(50, 5, 100, 1, 0, 3, "", "", "%");
	QM_Threshold_upper.Init(70, 5, 100, 1, 0, 3, "", "", "%");
	
//BF del	Math_Offset.Init(0.00000001, 0.00000001, 10, 0.000000005, 2, 3, "", "", "V");
	
	Math_Offset_mul.Init(0.001, 0.001, 1000, 0.001, 2, 3, "", "", "V");
	Math_Offset_add.Init(0.001, 0.001, 1000, 0.001, 2, 3, "", "", "V");
	Math_Offset_sub.Init(0.001, 0.001, 1000, 0.001, 2, 3, "", "", "V");

	Zero_Level1_Str.Init(0.00000001, 0.00000001, 10, 0.000000005, 2, 3, "", "Ch(1) = ", "V");
	Zero_Level2_Str.Init(0.00000001, 0.00000001, 10, 0.000000005, 2, 3, "", "Ch(2) = ", "V");
	Zero_Level3_Str.Init(0.00000001, 0.00000001, 10, 0.000000005, 2, 3, "", "Ch(3) = ", "V");
	Zero_Level4_Str.Init(0.00000001, 0.00000001, 10, 0.000000005, 2, 3, "", "Ch(4) = ", "V");
	
	Trigger_Level_Str.Init(0.00000001, 0.00000001, 10, 0.000000005, 2, 3, "", "", "V");
	Trigger_ExtLevel_Str.Init(0.00000001, 0.00000001, 10, 0.000000005, 2, 3, "", "", "V");	
	Trigger_Offset_Str.Init(0.00000001, 0.00000001, 10, 0.000000005, 2, 3, "", "", "s");	
	
	Timebase_Offset_Str.Init(0.00000001, 0.00000001, 10, 0.000000005, 2, 3, "", "", "s");

	HoldOff.Init(0.00000004, 0.00000004, 300, 0.00000004, 2, 5, "", "", "s");  // start 10 ns , min 10 ns, max 274 s, steps 8 ns , 2 afterdot, 4 maxlength, '' Text, '' PreText, 'S' Unit
	
	//BF obsolete FFT_Scale_Str.Init(0.00000004, 0.00000004, 300, 0.00000004, 2, 5, "", "", "dBV/");
	//BF obsolete FFT_Offset_Str.Init(0.00000004, 0.00000004, 300, 0.00000004, 2, 5, "", "", "dBV");
	
	switch(HoldOff_Expo)
	{
		case 1:
		{
			HoldOff.Write_StepSize(0.00000004);
			HoldOff.Write_Value((float) HoldOff_Value / (float) 1000000000);
			break;
		}
		case 2:
		{
			HoldOff.Write_StepSize(0.0000001);
			HoldOff.Write_Value((float) HoldOff_Value / (float) 1000000);
			break;
		}
		case 3:
		{
			HoldOff.Write_StepSize(0.001);
			HoldOff.Write_Value((float) HoldOff_Value / (float) 1000);
			break;
		}
		case 4:
		{
			HoldOff.Write_StepSize(1);
			HoldOff.Write_Value((float) HoldOff_Value);
			break;
		}
	}

	HoldOff.RenderText();	
	if (HoldOff_Value) trig_holdoff_reg = (unsigned long) (HoldOff.Read_Value() / (float) 0.000000008);
	else trig_holdoff_reg = 0x00000000;
		
		
	QM_Threshold_lower.Write_Value(20);	
	QM_Threshold_lower.RenderText();
	
	QM_Threshold_middle.Write_Value(50);	
	QM_Threshold_middle.RenderText();
	
	QM_Threshold_upper.Write_Value(80);	
	QM_Threshold_upper.RenderText();   	

	
	//Math_Offset_mul.Write_StepSize(1);
	Math_Offset_mul.Write_Value((float)((float)Math_Mul_Offset / (float)50000) );
	Math_Offset_mul.RenderText();
	//Math_Offset_sub.Write_StepSize(1);
	Math_Offset_sub.Write_Value((float)((float)Math_Sub_Offset / (float)50000) );
	Math_Offset_sub.RenderText();
	//Math_Offset_add.Write_StepSize(1);
	Math_Offset_add.Write_Value((float)((float)Math_Add_Offset / (float)50000) );
	Math_Offset_add.RenderText();
 

/*BF del
	if (config_loaded == false)
	{
//#ifdef _Debug_	
		printf("Generating Flash\n");
//#endif			
		// Generate Logo
				
		// Clear All Planes
		for (x = 0; x < 153600; x++) Planes[x] = 0;		

        	dma->np_piodata = 1;						// enable DMA Transfer
	   	dma->np_piodata = 0;

		//Generate_FLASH();
		
	    	// Write Config to Flash
	   	config_changed = true;
	    	AMDFlash::Write_Config_Flash();		
//#ifdef _Debug_		
		printf("Flash Generated\n");
		
		//printf("Resetting Hardware\n");		
//#endif		
		// Reset
//		Hardware::Reset_the_Watchdog();
		config_loaded = true;
		nr_delay(3000);
	}
*/	

	
    // execute config functions

//BF del not used	CALCTIMEBASE();
	
	CALCTRIGGER();
	CALCPRETRIGGER();
	
	if(FFT_Mode != FFT_OFF)
	FFT_CalcCursorData();
	else
	CALCCURSORDATA();

	CALCQMDATA();
#ifdef _Debug_
//	printf("Init : df : %f c1 : %d c2 : %d cs %d st %d sfi %d dst %d dsfi %d dwop %d\n", dmode_factor, Cursor_Delayed_1, Cursor_Delayed_2, Cursor_Delayed_Size, Selected_Timebase, SIGNALFaktor_idx, dmode_Selected_Timebase, dmode_SIGNALFaktor_idx, dmode_Window_Offset_Pos);
#endif	
	Hardware::Send_LED();
	
	if (Channel_1_Active) Hardware::UpdateChannel(1, 1);
	else
    {
        Hardware::UpdateChannel(1, 0);
        Channel_1_Active_Old = 1;
        VoltageChangedCh1 = 1;
    }

	if (Channel_2_Active) Hardware::UpdateChannel(2, 1);
	else
    {
        Hardware::UpdateChannel(2, 0);
        Channel_2_Active_Old = 1;
        VoltageChangedCh2 = 1;
    }

	if (Channel_3_Active) Hardware::UpdateChannel(3, 1);
	else
    {
        Hardware::UpdateChannel(3, 0);
        Channel_3_Active_Old = 1;
        VoltageChangedCh3 = 1;
    }

	if (Channel_4_Active) Hardware::UpdateChannel(4, 1);
	else
    {
        Hardware::UpdateChannel(4, 0);
        Channel_4_Active_Old = 1;
        VoltageChangedCh4 = 1;
    }

	Rotary_Steps = 0;
	Hardware::ON_Zero_Channel_1();
	Hardware::SetSwitches(1, Selected_Voltage_CH1);
	Hardware::SetDacOffset(1);
	Hardware::SetCHDacOffset(1);
	
	Rotary_Steps = 0;
	Hardware::ON_Zero_Channel_2();
	Hardware::SetSwitches(2, -1); // zwecks andere daten -switches2 in config
	Hardware::SetDacOffset(2);
	Hardware::SetCHDacOffset(2);
	
	if (NumberOfChannels > 2)
	{
        	Rotary_Steps = 0;
        	Hardware::ON_Zero_Channel_3();
		Hardware::SetSwitches(3, Selected_Voltage_CH3);
		Hardware::SetDacOffset(3);
		Hardware::SetCHDacOffset(3);
		
		Rotary_Steps = 0;
		Hardware::ON_Zero_Channel_4();
		Hardware::SetSwitches(4, Selected_Voltage_CH4);
		Hardware::SetDacOffset(4);
		Hardware::SetCHDacOffset(4);
	}
		
		
	data_adr->np_piodata = ram_adress_reg;	

//	printf("Math aktiv %d - %d\n", Channel_Math_Active, NumberOfChannels);
	
	if (Channel_Math_Active == 1)
	{
		if (NumberOfChannels == 2)
		{
		//if (MenuPopupStatus[1][2] == 1) MenuPopupStatus[1][2] = 2;
		//if (MenuPopupStatus[3][2] == 1) MenuPopupStatus[3][2] = 2;
		}
		else
		{
		//if (MenuPopupStatus[1][4] == 1) MenuPopupStatus[1][4] = 2;
		//if (MenuPopupStatus[3][4] == 1) MenuPopupStatus[3][4] = 2;
		}
	}

	DRAW_SPLASH(true);
	
	//ColorDemo2();	

	// Draw Startscreen
	DrawInitialScreen();
	
	TimebaseChanged     = 1;
	TimeOffsetChanged   = 1;
	TriggerLevelChanged = 1;
	TriggerModeChanged  = 1;
	TriggerWayChanged   = 1;

	StatusUpdate();
	
	CursorChanged = 3;
	Cursor_Data_First_Draw = 1;
	DRAWCURSORDATA(0);
	
	Quick_Measure_First_Draw = 1;
	DRAWQMDATA(0); 	
	
	DRAW_ALL_CURSOR();
#ifdef _Debug_    		
	if (Debug_Mode) printf("Flash Drawed\n");					
#endif

 /* TMW    if (MenuStatus[5][4] == 241) DRAWMEMORY(0, 1, 1);  TMW */  		//BF memory browse		
	
	// must be set bevor SetupADC						
	init_done   = true;	
	first_start = true;
	Hardware::UpdateTrigger(30);
	first_start = false; 	
	
	serdata->np_piodata = ext_trg_val_reg;

	serstartpwm->np_piodata = 1;
	serstartpwm->np_piodata = 0;	
	
	
		/*
	send_buffer[0] = 3;						// Command Write Data
	send_buffer[1] = 0;
						
	Hardware::SendData(send_buffer);	
	*/

}

//##########################################################################################################################################################

void Display::RefreshScreen(void)
{
	unsigned long lX;

// BF delete old cursors and markers
	//Grid area
	for (lX = GRID_PLANE_MIN; lX < GRID_PLANE_MAX; lX++)
	{
		Buffer_Mark1Plane[lX] = 0;
		Marker_Plane1[lX] = 0;
	}	


	//------------------------------------------------------------------------------
	//Draw Grids		//BF
	//------------------------------------------------------------------------------
	if (XY_Mode)								// Check if XY-mode
	GRIDXY(GRID_XY_XOFFS, GRID_YOFFS, Grid_Plane);				// Draw XY grid into gridplane
	else if (MenuStatus[MENU_TIMEBASE][1] == 1) 				// delayed mode
	GRIDDELAYED(GRID_XOFFS, GRID_YOFFS, Grid_Plane);
	else if (FFT_Mode != FFT_OFF) 						// FFT mode
	GRIDFFT(GRID_XOFFS_FFT, GRID_YOFFS_FFT, Grid_Plane);			// Draw FFT grid into gridplane
	else 									// main mode
	GRID(GRID_XOFFS, GRID_YOFFS, Grid_Plane);				// Draw main grid into gridplane
	

	VoltageChangedCh1 = 1;	
	VoltageChangedCh2 = 1;	
	if(NumberOfChannels == 4)
	{
		VoltageChangedCh3 = 1;	
		VoltageChangedCh4 = 1;	
	}	

	//Print text into the top area
	StatusUpdate();

	FFT_DrawStatus(0);		//draw status if FFT is active

}
//##########################################################################################################################################################

void Display::REMOVE_SPLASH(void)
{
	unsigned long x;
	char BufText[80];
	char BufText2[80];	
	char BufText3[80];	
	char BufText4[80];
	
	sprintf(BufText, "%1i.%1i.BF.%1i.%1i beta", tc_version, tc_subversion, tc_dev_version, tc_dev_subversion);
	sprintf(BufText2, "%X.%c%c", tc_hw_version, tc_production_lot1, tc_production_lot2);
	sprintf(BufText3, "%06d%02d%1X%1X", tc_serial, ((tc_shipment_date & 0x0000FF00) >> 8), ((tc_shipment_date & 0x000000F0) >> 4), (tc_shipment_date & 0x0000000F));	
	sprintf(BufText4, "W%04dA", tc_model);
		
	// Clear Buffer Planes
	for (x = 0; x < 9600; x++)
	{
		Buffer_Plane1[x] = 0;
		Buffer_Plane2[x] = 0;
		Buffer_Plane3[x] = 0;
		Buffer_Plane4[x] = 0;
		Buffer_PlaneM[x] = 0;
		//Buffer_UI2Plane[x] = 0;		 is in marker 1
		Buffer_Mark1Plane[x] = 0;
	}

	for (x = 0; x < 9600; x++)
	{
		UI_Plane5[x] = 0;
	}	  	
			
	BITBITMAP(bitbmWTAG, Splash_Logo_PosX, Splash_Logo_PosY, 0, Grid_Plane);
			
	//TEXTOUTvbig("Developed by", Splash_Data_PosX + 28, Splash_Data_PosY - 20, 0, UI_Plane1);		
	TEXTOUTxvbig("Hayo W. (BlueFlash)", Splash_Logo_PosX - 14, Splash_Data_PosY - 20, 0, UI_Plane1);
	
	TEXTOUTvbig("Model : ", Splash_Data_PosX + 28, Splash_Data_PosY + 50, 0, UI_Plane1);
	TEXTOUTvbig(BufText4, Splash_Data_PosX + 148, Splash_Data_PosY + 50, 0, UI_Plane1);	
		
	TEXTOUTvbig("Hardware Version : ", Splash_Data_PosX + 28, Splash_Data_PosY + 70, 0, UI_Plane1);
	TEXTOUTvbig(BufText2, Splash_Data_PosX + 148, Splash_Data_PosY + 70, 0, UI_Plane1);		
	
	TEXTOUTvbig("Software Version : ", Splash_Data_PosX + 28, Splash_Data_PosY + 90, 0, UI_Plane1);
	TEXTOUTvbig(BufText, Splash_Data_PosX + 148, Splash_Data_PosY + 90, 0, UI_Plane1);	
		
	TEXTOUTvbig("Serial Number : ", Splash_Data_PosX + 28, Splash_Data_PosY + 110, 0, UI_Plane1);
	TEXTOUTvbig(BufText3, Splash_Data_PosX + 148, Splash_Data_PosY + 110, 0, UI_Plane1);	 	
		
	TEXTOUTvbig("No Rights Reserved _ 2009", Splash_Data_PosX + 236, Splash_Data_PosY + 250, 0, UI_Plane1);		
			
	GridColor_Val = GridColorArray[GridColorArrayPos];
	Hardware::SetupADC();
#ifdef _Debug_	
	if (Debug_Mode) printf("Remove Splash finished\n");	
#endif	
	Splash_removed = true;
}

//##########################################################################################################################################################

void Display::TEXTOUTvbig(char *String, int x, int y, char set, unsigned long *layer)	// Draw Text at Pos X / Y with Color and BackColor (FF = Transparent)
{
	int Buf, Ct, Ctt, Bit;
	int count;
	
	count = strlen(String);

	// Start drawing background
	Bit = 0;									// Correction Counter for different letter size (e.g. i or w)
	
	for (Ct = 0;Ct < count; Ct++)							// Calculate the correction for the background
	{
		Buf = String[Ct] - 32;							// get the letter
		
		for (Ctt = 0;Ctt < 14; Ctt++)						// test the Bytes of the letter (x size)
		{
			if (FONT10X9[Buf][(Ctt * 2) + 1] == 0x80) Bit++;		// if itŽs size small than the minimum size 0x80 signals it
		}
	}	
	
	//Draw Background	
//	if (backcolor != 0xFF) DRAWP(x - 3, y - 4, (14 * count) + 5 - Bit, 18, backcolor, layer);	// Text background (0xFF transparent) (Lettercount * Minimumsize) - Lettersize correktion
	
	//Start drawing letters
	Bit = 0;									// Correction Counter for different letter size (e.g. i or w)
	
	for (Ct = 0;Ct < count; Ct++)							// Letterloop
	{
		Buf = String[Ct] - 32;							// get the letter
		
		for (Ctt = 0;Ctt < 14; Ctt++)						// Letterbyteloop
		{		
			if (FONT10X9[Buf][(Ctt * 2) + 1] == 0x80) Bit++;	// if Letterbyte = 0x80 the end of the letter is reached and the next letter not move forward
			else
			{
				if ((FONT10X9[Buf][(Ctt * 2)] & 0x01) == 1) PIXELP(x + Ctt + (Ct * 14) - Bit, y, set, layer);			// If Bit 0 set Pixel 0 in the Row is drawed in the color
				if ((FONT10X9[Buf][(Ctt * 2)] & 0x02) == 2) PIXELP(x + Ctt + (Ct * 14) - Bit, y + 1, set, layer);		// If Bit 1 set Pixel 1 in the Row is drawed in the color
				if ((FONT10X9[Buf][(Ctt * 2)] & 0x04) == 4) PIXELP(x + Ctt + (Ct * 14) - Bit, y + 2, set, layer);		// If Bit 2 set Pixel 2 in the Row is drawed in the color
				if ((FONT10X9[Buf][(Ctt * 2)] & 0x08) == 8) PIXELP(x + Ctt + (Ct * 14) - Bit, y + 3, set, layer);		// If Bit 3 set Pixel 3 in the Row is drawed in the color
				if ((FONT10X9[Buf][(Ctt * 2)] & 0x10) == 16) PIXELP(x + Ctt + (Ct * 14) - Bit, y + 4, set, layer);		// If Bit 4 set Pixel 4 in the Row is drawed in the color
				if ((FONT10X9[Buf][(Ctt * 2)] & 0x20) == 32) PIXELP(x + Ctt + (Ct * 14) - Bit, y + 5, set, layer);		// If Bit 5 set Pixel 5 in the Row is drawed in the color
				if ((FONT10X9[Buf][(Ctt * 2)] & 0x40) == 64) PIXELP(x + Ctt + (Ct * 14) - Bit, y + 6, set, layer);		// If Bit 6 set Pixel 6 in the Row is drawed in the color
				if ((FONT10X9[Buf][(Ctt * 2)] & 0x80) == 128) PIXELP(x + Ctt + (Ct * 14) - Bit, y + 7, set, layer);		// If Bit 7 set Pixel 7 in the Row is drawed in the color
				if ((FONT10X9[Buf][(Ctt * 2) + 1] & 0x01) == 1) PIXELP(x + Ctt + (Ct * 14) - Bit, y + 8, set, layer);		// If Bit 8 set Pixel 8 in the Row is drawed in the color
				if ((FONT10X9[Buf][(Ctt * 2) + 1] & 0x02) == 2) PIXELP(x + Ctt + (Ct * 14) - Bit, y + 9, set, layer);		// If Bit 9 set Pixel 9 in the Row is drawed in the color
				if ((FONT10X9[Buf][(Ctt * 2) + 1] & 0x04) == 4) PIXELP(x + Ctt + (Ct * 14) - Bit, y + 10, set, layer);		// If Bit 10 set Pixel 10 in the Row is drawed in the color
				if ((FONT10X9[Buf][(Ctt * 2) + 1] & 0x08) == 8) PIXELP(x + Ctt + (Ct * 14) - Bit, y + 11, set, layer);		// If Bit 11 set Pixel 11 in the Row is drawed in the color
				if ((FONT10X9[Buf][(Ctt * 2) + 1] & 0x10) == 16) PIXELP(x + Ctt + (Ct * 14) - Bit, y + 12, set, layer);		// If Bit 12 set Pixel 12 in the Row is drawed in the color
				if ((FONT10X9[Buf][(Ctt * 2) + 1] & 0x20) == 32) PIXELP(x + Ctt + (Ct * 14) - Bit, y + 13, set, layer);		// If Bit 13 set Pixel 13 in the Row is drawed in the color
				if ((FONT10X9[Buf][(Ctt * 2) + 1] & 0x40) == 64) PIXELP(x + Ctt + (Ct * 14) - Bit, y + 14, set, layer);		// If Bit 14 set Pixel 14 in the Row is drawed in the color
				if ((FONT10X9[Buf][(Ctt * 2) + 1] & 0x80) == 128) PIXELP(x + Ctt + (Ct * 14) - Bit, y + 15, set, layer);	// If Bit 15 set Pixel 15 in the Row is drawed in the color
			}							
		}
	}
}
//##########################################################################################################################################################

void Display::TEXTOUTxvbig(char *String, int x, int y, char set, unsigned long *layer)	// Draw Text at Pos X / Y with Color and BackColor (FF = Transparent)
{
	int Ct, Ctt, Bit;
	int count;
	unsigned char Buf;
	
	count = strlen(String);
	
	// Start drawing background
	Bit = 0;									// Correction Counter for different letter size (e.g. i or w)
	
	for (Ct = 0;Ct < count; Ct++)							// Calculate the correction for the background
	{
		Buf = String[Ct] - 32;							// get the letter
				
		for (Ctt = 0;Ctt < 14; Ctt++)						// test the Bytes of the letter (x size)
		{
			if (FONT12X9[Buf][(Ctt * 2) + 1] == 0x80) Bit++;		// if it's size small than the minimum size 0x80 signals it		
		}
	}	
	
	//Draw Background	
//	if (backcolor != 0xFF) DRAWP(x - 3, y - 3, (14 * count) + 5 - Bit, 19, backcolor, layer);	// Text background (0xFF transparent) (Lettercount * Minimumsize) - Lettersize correktion
	
	//Start drawing letters
	Bit = 0;													// Correction Counter for different letter size (e.g. i or w)
	
	for (Ct = 0;Ct < count; Ct++)								// Letterloop
	{
		Buf = String[Ct] - 32;								// get the letter
		for (Ctt = 0;Ctt < 14; Ctt++)							// Letterbyteloop
		{
			if (FONT12X9[Buf][(Ctt * 2) + 1] == 0x80) Bit++;	// if Letterbyte = 0x80 the end of the letter is reached and the next letter not move forward
			else
			{
				if ((FONT12X9[Buf][(Ctt * 2)] & 0x01) == 1) PIXELP(x + Ctt + (Ct * 14) - Bit, y, set, layer);					// If Bit 0 set Pixel 0 in the Row is drawed in the color
				if ((FONT12X9[Buf][(Ctt * 2)] & 0x02) == 2) PIXELP(x + Ctt + (Ct * 14) - Bit, y + 1, set, layer);				// If Bit 1 set Pixel 1 in the Row is drawed in the color
				if ((FONT12X9[Buf][(Ctt * 2)] & 0x04) == 4) PIXELP(x + Ctt + (Ct * 14) - Bit, y + 2, set, layer);				// If Bit 2 set Pixel 2 in the Row is drawed in the color
				if ((FONT12X9[Buf][(Ctt * 2)] & 0x08) == 8) PIXELP(x + Ctt + (Ct * 14) - Bit, y + 3, set, layer);				// If Bit 3 set Pixel 3 in the Row is drawed in the color
				if ((FONT12X9[Buf][(Ctt * 2)] & 0x10) == 16) PIXELP(x + Ctt + (Ct * 14) - Bit, y + 4, set, layer);				// If Bit 4 set Pixel 4 in the Row is drawed in the color
				if ((FONT12X9[Buf][(Ctt * 2)] & 0x20) == 32) PIXELP(x + Ctt + (Ct * 14) - Bit, y + 5, set, layer);				// If Bit 5 set Pixel 5 in the Row is drawed in the color
				if ((FONT12X9[Buf][(Ctt * 2)] & 0x40) == 64) PIXELP(x + Ctt + (Ct * 14) - Bit, y + 6, set, layer);				// If Bit 6 set Pixel 6 in the Row is drawed in the color
				if ((FONT12X9[Buf][(Ctt * 2)] & 0x80) == 128) PIXELP(x + Ctt + (Ct * 14) - Bit, y + 7, set, layer);				// If Bit 7 set Pixel 7 in the Row is drawed in the color
				if ((FONT12X9[Buf][(Ctt * 2) + 1] & 0x01) == 1) PIXELP(x + Ctt + (Ct * 14) - Bit, y + 8, set, layer);			// If Bit 8 set Pixel 8 in the Row is drawed in the color
				if ((FONT12X9[Buf][(Ctt * 2) + 1] & 0x02) == 2) PIXELP(x + Ctt + (Ct * 14) - Bit, y + 9, set, layer);			// If Bit 9 set Pixel 9 in the Row is drawed in the color
				if ((FONT12X9[Buf][(Ctt * 2) + 1] & 0x04) == 4) PIXELP(x + Ctt + (Ct * 14) - Bit, y + 10, set, layer);			// If Bit 10 set Pixel 10 in the Row is drawed in the color
				if ((FONT12X9[Buf][(Ctt * 2) + 1] & 0x08) == 8) PIXELP(x + Ctt + (Ct * 14) - Bit, y + 11, set, layer);			// If Bit 11 set Pixel 11 in the Row is drawed in the color
				if ((FONT12X9[Buf][(Ctt * 2) + 1] & 0x10) == 16) PIXELP(x + Ctt + (Ct * 14) - Bit, y + 12, set, layer);		// If Bit 12 set Pixel 12 in the Row is drawed in the color
				if ((FONT12X9[Buf][(Ctt * 2) + 1] & 0x20) == 32) PIXELP(x + Ctt + (Ct * 14) - Bit, y + 13, set, layer);		// If Bit 13 set Pixel 13 in the Row is drawed in the color
				if ((FONT12X9[Buf][(Ctt * 2) + 1] & 0x40) == 64) PIXELP(x + Ctt + (Ct * 14) - Bit, y + 14, set, layer);		// If Bit 14 set Pixel 14 in the Row is drawed in the color
				if ((FONT12X9[Buf][(Ctt * 2) + 1] & 0x80) == 128) PIXELP(x + Ctt + (Ct * 14) - Bit, y + 15, set, layer);		// If Bit 15 set Pixel 15 in the Row is drawed in the color
			}								
		}
	}
}
