#include "hardware_t.h"
#include "tc_vars.h"
#include "flash_t.h"
#include "display_t.h"
//#include "math.h"

// Class Display
//##########################################################################################################################################################

void Display::BITBITMAP(unsigned char *pointer, int x, int y, unsigned char set, unsigned long *layer)	// Draw two color bitmap
{
	int cc, cx, cy, pos;
	unsigned char ccmask;

	int sizex = (int) pointer[0] / 8;
	int sizey = pointer[1];


	pos = 2;

	for (cy = 0; cy < sizey; cy++)
	{
		for (cx = 0; cx < sizex; cx++)
		{
			ccmask = 0x80;			
			
			for (cc = 0; cc < 8; cc++)
			{			
				if ((pointer[pos] & ccmask) == ccmask)
				{
					PIXELP(x + cc + (cx * 8), y + cy, set, layer);
				}
				ccmask = ccmask >> 1;
			}
			pos++;			
		}
	}
}
//##########################################################################################################################################################

void Display::CLEAR2BYTE(long addressold)
{
    asm("
  		   PFX      %hi(0x00)
  		   MOVI     %l1, %lo(0x00)
           FILL8	%r0, %l1				
           ST8S		[%i0], %r0, 2

  		   PFX      %hi(0x00)
  		   MOVI     %l1, %lo(0x00)
           FILL8	%r0, %l1				
           ST8S		[%i0], %r0, 3
		");	
}
//##########################################################################################################################################################

void Display::WRITE2BYTE(unsigned char byte1, unsigned char byte2, long addressnew)
{
    asm("
           FILL8	%r0, %i1             			
           ST8S		[%i2], %r0, 2
  				
           FILL8	%r0, %i0             			
           ST8S		[%i2], %r0, 3
		");	
}
//##########################################################################################################################################################

void Display::OVERWRITE2BYTE(unsigned char byte1, unsigned char byte2, long addressnew)
{
    asm("
           LD       %r0, [%i2]
           EXT8S    %r0, 2
           OR       %r0, %i1
           FILL8	%r0, %r0
           ST8S		[%i2], %r0, 2

           LD       %r0, [%i2]
           EXT8S    %r0, 3
           OR       %r0, %i0
           FILL8	%r0, %r0             			
           ST8S		[%i2], %r0, 3
		");	
}
//##########################################################################################################################################################

void Display::CLEAR2BYTEBACK(long addressold)
{
    asm("
  		   PFX      %hi(0x00)
  		   MOVI     %l1, %lo(0x00)
           FILL8	%r0, %l1				
           ST8S		[%i0], %r0, 0

  		   PFX      %hi(0x00)
  		   MOVI     %l1, %lo(0x00)
           FILL8	%r0, %l1				
           ST8S		[%i0], %r0, 1
		");	
}
//##########################################################################################################################################################

void Display::WRITE2BYTEBACK(unsigned char byte1, unsigned char byte2, long addressnew)
{
    asm("
           FILL8	%r0, %i1             			
           ST8S		[%i2], %r0, 0
  				
           FILL8	%r0, %i0             			
           ST8S		[%i2], %r0, 1
		");	
}

//##########################################################################################################################################################
/* BF del not used
void Display::Clear_Flips(void)
{
    return;

    if (screen_sel == 0)
    {
        if (NumberOfChannels == 2) Hardware::Planes_clear_asm_all(3, 0x009311F0, 0x0093A7F0, 0x009569F0, 0x00000000, 0x00000000);
        else Hardware::Planes_clear_asm_all(5, 0x009311F0, 0x0093A7F0, 0x00943DF0, 0x0094D3F0, 0x009569F0);
    }
    else
    {
        if (NumberOfChannels == 2) Hardware::Planes_clear_asm_all(3, 0x0098EDF0, 0x009983F0, 0x009B45F0, 0x00000000, 0x00000000);
        else Hardware::Planes_clear_asm_all(5, 0x0098EDF0, 0x009983F0, 0x009A19F0, 0x009AAFF0, 0x009B45F0);
    }
    if (Debug_Mode) printf("clear flips\n");
} */
//##########################################################################################################################################################
// BF function for sending float values via printf() to RS232, resolving the problem of systemcrash in open source FW
// when using the printf() directly

void Display::floatprintf(float fvalue)
{
	long int pre, aft,factor;
	int i;
	
	if (fvalue == 0)
	{ printf("0.000"); return;}

	//calculating pre FP value
	pre = (long int)fvalue;
	if(pre == 0 && fvalue < 0)	//negativ
		printf("-%d.",pre);
	else
		printf("%d.",pre);
		
	//get leading zero
	if((fvalue - pre) != 0) //check if numbers after FP exist
	{
		for(factor = 10, i= 0 ;0 == (long int)((fvalue - pre) * factor) && i<11; factor*=10, i++)
		{ printf("0"); }
	}
	else
	{	
		printf("000");
		return;
	}
	
	//calc after FP value, minimum 3 numbers after FP
	aft = (long int)((fvalue - pre) * factor * 100);
	
	if(aft < 0)	//kill sign
	{ aft *= -1; }

	printf("%d",aft);

	if(i >= 11)
	printf("overflow");

}
//##########################################################################################################################################################

void Display::PIXELP(int x, int y, char set, unsigned long *planeaddress)
{
	int pos;
	int bit;
	int intbuf;
	
	intbuf = x >> 5;	
	pos = Display_Line_Adresses[y] + intbuf;		
	bit = x - (intbuf << 5);
	
	if (set == 1) *(planeaddress + pos) |= BitMasks2[bit];
	else *(planeaddress + pos) &= BitMasks[bit];
}
//##########################################################################################################################################################

void Display::RemoveStatusBtn(void)
{
	switch(StatusbarChanged)
	{
		case 1:
		{
			DRAWROUNDBUTTON(15, 0, 64, 20, btnUnpushed, btnRemove);						// Remove Button
			break;
		}
		case 2:
		{
			DRAWROUNDBUTTON(91, 0, 64, 20, btnUnpushed, btnRemove);						// Remove Button
			break;
		}
		case 3:
		{
			DRAWROUNDBUTTON(167, 0, 64, 20, btnUnpushed, btnRemove);						// Remove Button
			break;
		}
		case 4:
		{
			DRAWROUNDBUTTON(243, 0, 64, 20, btnUnpushed, btnRemove);						// Remove Button
			break;
		}	

		case 6:
		{
			DRAWROUNDBUTTON(558, 0, 82, 20, btnUnpushed, btnRemove);						// Remove Button
			break;
		}			
		case 10: // Timebase
		{
			DRAWROUNDBUTTON(404, 0, 64, 20, btnUnpushed, btnRemove);						// Remove Button
			break;
		}
		case 11: // Timebase Delayed
		{
			DRAWROUNDBUTTON(326, 0, 64, 20, btnUnpushed, btnRemove);						// Remove Button
			break;
		}							
		
	}

	StatusbarChanged = 0;	
}
//##########################################################################################################################################################

void Display::RemoveZeroSign(long addressold)
{
    CLEAR2BYTE(addressold + 0x000);
    CLEAR2BYTE(addressold + 0x050);
    CLEAR2BYTE(addressold + 0x0A0);
    CLEAR2BYTE(addressold + 0x0F0);
    CLEAR2BYTE(addressold + 0x140);
    CLEAR2BYTE(addressold + 0x190);
    CLEAR2BYTE(addressold + 0x1E0);
    CLEAR2BYTE(addressold + 0x230);
    CLEAR2BYTE(addressold + 0x280);
    CLEAR2BYTE(addressold + 0x2D0);
    CLEAR2BYTE(addressold + 0x320);
    CLEAR2BYTE(addressold + 0x370);
    CLEAR2BYTE(addressold + 0x3C0);
    CLEAR2BYTE(addressold + 0x410);
    CLEAR2BYTE(addressold + 0x460);
    CLEAR2BYTE(addressold + 0x4B0);
}
//##########################################################################################################################################################

void Display::DrawZeroSign(unsigned char *sign, long addressnew)
{
    WRITE2BYTE(sign[2], sign[3], addressnew + 0x000);
    WRITE2BYTE(sign[4], sign[5], addressnew + 0x050);
    WRITE2BYTE(sign[6], sign[7], addressnew + 0x0A0);
    WRITE2BYTE(sign[8], sign[9], addressnew + 0x0F0);
    WRITE2BYTE(sign[10], sign[11], addressnew + 0x140);
    WRITE2BYTE(sign[12], sign[13], addressnew + 0x190);
    WRITE2BYTE(sign[14], sign[15], addressnew + 0x1E0);
    WRITE2BYTE(sign[16], sign[17], addressnew + 0x230);
    WRITE2BYTE(sign[18], sign[19], addressnew + 0x280);
    WRITE2BYTE(sign[20], sign[21], addressnew + 0x2D0);
    WRITE2BYTE(sign[22], sign[23], addressnew + 0x320);
    WRITE2BYTE(sign[24], sign[25], addressnew + 0x370);
    WRITE2BYTE(sign[26], sign[27], addressnew + 0x3C0);
    WRITE2BYTE(sign[28], sign[29], addressnew + 0x410);
    WRITE2BYTE(sign[30], sign[31], addressnew + 0x460);
    WRITE2BYTE(sign[32], sign[33], addressnew + 0x4B0);
}
//##########################################################################################################################################################

void Display::DrawZeroSignXY(int position, char set)
{
    int posiy = 20;

    DRAWLINEP(position + 80, posiy, position + 80, posiy + 6, set, Buffer_Plane1);

    DRAWLINEP(position + 80 - 1, posiy + 1, position + 80 + 2, posiy + 1, set, Buffer_Plane1);
    DRAWLINEP(position + 80 - 2, posiy + 2, position + 80 + 3, posiy + 2, set, Buffer_Plane1);
    DRAWLINEP(position + 80 - 3, posiy + 3, position + 80 + 4, posiy + 3, set, Buffer_Plane1);

    DRAWLINEP(position + 80 - 2, posiy + 6, position + 80 + 3, posiy + 6, set, Buffer_Plane1);
    DRAWLINEP(position + 80 - 1, posiy + 8, position + 80 + 2, posiy + 8, set, Buffer_Plane1);
    PIXELP(position + 80, posiy + 10, set, Buffer_Plane1);

    PIXELP(position + 80 - 7, posiy + 3, set, Buffer_Plane1);
    PIXELP(position + 80 - 6, posiy + 2, set, Buffer_Plane1);
    DRAWLINEP(position + 80 - 5, posiy + 1, position + 80 - 5, posiy + 7, set, Buffer_Plane1);
}
//##########################################################################################################################################################

void Display::OverDrawZeroSign(unsigned char *sign, long addressnew)
{
    OVERWRITE2BYTE(sign[2], sign[3], addressnew + 0x000);
    OVERWRITE2BYTE(sign[4], sign[5], addressnew + 0x050);
    OVERWRITE2BYTE(sign[6], sign[7], addressnew + 0x0A0);
    OVERWRITE2BYTE(sign[8], sign[9], addressnew + 0x0F0);
    OVERWRITE2BYTE(sign[10], sign[11], addressnew + 0x140);
    OVERWRITE2BYTE(sign[12], sign[13], addressnew + 0x190);
    OVERWRITE2BYTE(sign[14], sign[15], addressnew + 0x1E0);
    OVERWRITE2BYTE(sign[16], sign[17], addressnew + 0x230);
    OVERWRITE2BYTE(sign[18], sign[19], addressnew + 0x280);
    OVERWRITE2BYTE(sign[20], sign[21], addressnew + 0x2D0);
    OVERWRITE2BYTE(sign[22], sign[23], addressnew + 0x320);
    OVERWRITE2BYTE(sign[24], sign[25], addressnew + 0x370);
    OVERWRITE2BYTE(sign[26], sign[27], addressnew + 0x3C0);
    OVERWRITE2BYTE(sign[28], sign[29], addressnew + 0x410);
    OVERWRITE2BYTE(sign[30], sign[31], addressnew + 0x460);
    OVERWRITE2BYTE(sign[32], sign[33], addressnew + 0x4B0);
}
//##########################################################################################################################################################

void Display::RemoveZeroSignBack(long addressold)
{
    CLEAR2BYTEBACK(addressold + 0x04F);
    CLEAR2BYTEBACK(addressold + 0x09F);
    CLEAR2BYTEBACK(addressold + 0x0EF);
    CLEAR2BYTEBACK(addressold + 0x13F);
    CLEAR2BYTEBACK(addressold + 0x18F);
    CLEAR2BYTEBACK(addressold + 0x1DF);
    CLEAR2BYTEBACK(addressold + 0x22F);
    CLEAR2BYTEBACK(addressold + 0x27F);
    CLEAR2BYTEBACK(addressold + 0x2CF);
    CLEAR2BYTEBACK(addressold + 0x31F);
    CLEAR2BYTEBACK(addressold + 0x36F);
    CLEAR2BYTEBACK(addressold + 0x3BF);
    CLEAR2BYTEBACK(addressold + 0x40F);
    CLEAR2BYTEBACK(addressold + 0x45F);
    CLEAR2BYTEBACK(addressold + 0x4AF);
    CLEAR2BYTEBACK(addressold + 0x4FF);
    CLEAR2BYTEBACK(addressold + 0x54F);
    CLEAR2BYTEBACK(addressold + 0x59F);
    CLEAR2BYTEBACK(addressold + 0x5EF);
    CLEAR2BYTEBACK(addressold + 0x63F);
    CLEAR2BYTEBACK(addressold + 0x68F);
    CLEAR2BYTEBACK(addressold + 0x6DF);
    CLEAR2BYTEBACK(addressold + 0x72F);
    CLEAR2BYTEBACK(addressold + 0x77F);
    CLEAR2BYTEBACK(addressold + 0x7CF);
    CLEAR2BYTEBACK(addressold + 0x81F);
    CLEAR2BYTEBACK(addressold + 0x86F);
    CLEAR2BYTEBACK(addressold + 0x8BF);
    CLEAR2BYTEBACK(addressold + 0x90F);
}
//##########################################################################################################################################################

void Display::DrawZeroSignBack(unsigned char *sign, long addressnew)
{
    WRITE2BYTEBACK(sign[2], sign[3], addressnew + 0x04F);
    WRITE2BYTEBACK(sign[4], sign[5], addressnew + 0x09F);
    WRITE2BYTEBACK(sign[6], sign[7], addressnew + 0x0EF);
    WRITE2BYTEBACK(sign[8], sign[9], addressnew + 0x13F);
    WRITE2BYTEBACK(sign[10], sign[11], addressnew + 0x18F);
    WRITE2BYTEBACK(sign[12], sign[13], addressnew + 0x1DF);
    WRITE2BYTEBACK(sign[14], sign[15], addressnew + 0x22F);
    WRITE2BYTEBACK(sign[16], sign[17], addressnew + 0x27F);
    WRITE2BYTEBACK(sign[18], sign[19], addressnew + 0x2CF);
    WRITE2BYTEBACK(sign[20], sign[21], addressnew + 0x31F);
    WRITE2BYTEBACK(sign[22], sign[23], addressnew + 0x36F);
    WRITE2BYTEBACK(sign[24], sign[25], addressnew + 0x3BF);
    WRITE2BYTEBACK(sign[26], sign[27], addressnew + 0x40F);
    WRITE2BYTEBACK(sign[28], sign[29], addressnew + 0x45F);
    WRITE2BYTEBACK(sign[30], sign[31], addressnew + 0x4AF);
    WRITE2BYTEBACK(sign[32], sign[33], addressnew + 0x4FF);
    WRITE2BYTEBACK(sign[34], sign[35], addressnew + 0x54F);
    WRITE2BYTEBACK(sign[36], sign[37], addressnew + 0x59F);
    WRITE2BYTEBACK(sign[38], sign[39], addressnew + 0x5EF);
    WRITE2BYTEBACK(sign[40], sign[41], addressnew + 0x63F);
    WRITE2BYTEBACK(sign[42], sign[43], addressnew + 0x68F);
    WRITE2BYTEBACK(sign[44], sign[45], addressnew + 0x6DF);
    WRITE2BYTEBACK(sign[46], sign[47], addressnew + 0x72F);
    WRITE2BYTEBACK(sign[48], sign[49], addressnew + 0x77F);
    WRITE2BYTEBACK(sign[50], sign[51], addressnew + 0x7CF);
    WRITE2BYTEBACK(sign[52], sign[53], addressnew + 0x81F);
    WRITE2BYTEBACK(sign[54], sign[55], addressnew + 0x86F);
    WRITE2BYTEBACK(sign[56], sign[57], addressnew + 0x8BF);
    WRITE2BYTEBACK(sign[58], sign[59], addressnew + 0x90F);
}
//##########################################################################################################################################################
// BF changed 
void Display::StatusUpdate(void)
{
	//BF if ((Channel_1_Active == 0) && (Channel_1_Active_Old == 1) && (VoltageChangedCh1 == 1))
	if ((Channel_1_Active == 0)  && (VoltageChangedCh1 == 1))
	{
		DRAWP(6, 4, 6, 10, 0, Channel_Plane1);								// Clear Channel Text	
//		DRAWP(6, 4, 6, 10, 0, Buffer_Plane1);								// Clear Channel Text	
		DRAWP(3, 0, 68, 18, 0, UI_Plane2);								// Clear Text

		DRAWSQUAREBUTTON(2, 1, 14, 16, btnPushed, btnRemove);						// Button
		
	}
	//BF else if ((Channel_1_Active == 1) && (Channel_1_Active_Old == 0) && (VoltageChangedCh1 == 1))
	else if ((Channel_1_Active == 1) && (VoltageChangedCh1 == 1))
	{
		DRAWSQUAREBUTTON(2, 1, 14, 16, btnPushed, btnDraw);						// Button
		TEXTOUTvbig("1", 6, 4, 1, Channel_Plane1);							// Draw Voltage Text
//		TEXTOUTvbig("1", 6, 4, 1, Buffer_Plane1);							// Draw Voltage Text
		
		//VoltageChangedCh1 = 1;

	}
	
	//BF if ((Channel_2_Active == 0) && (Channel_2_Active_Old == 1) && (VoltageChangedCh2 == 1))
	if ((Channel_2_Active == 0) && (VoltageChangedCh2 == 1))
	{
		DRAWP(82, 4, 6, 10, 0, Channel_Plane2);								// Clear Channel Text	
//		DRAWP(82, 4, 6, 10, 0, Buffer_Plane2);								// Clear Channel Text	
		DRAWP(79, 0, 68, 18, 0, UI_Plane2);								// Clear Text

		DRAWSQUAREBUTTON(78, 1, 14, 16, btnPushed, btnRemove);						// Button
				

	}
	//BF else if ((Channel_2_Active == 1) && (Channel_2_Active_Old == 0) && (VoltageChangedCh2 == 1))
	else if ((Channel_2_Active == 1) && (VoltageChangedCh2 == 1))
	{
		DRAWSQUAREBUTTON(78, 1, 14, 16, btnPushed, btnDraw);						// Button
		TEXTOUTvbig("2", 82, 4, 1, Channel_Plane2);							// Draw Voltage Text
//		TEXTOUTvbig("2", 82, 4, 1, Buffer_Plane2);							// Draw Voltage Text
		
		//VoltageChangedCh2 = 1;
		
	}
	
	if (NumberOfChannels > 2)
	{
		//BF if ((Channel_3_Active == 0) && (Channel_3_Active_Old == 1) && (VoltageChangedCh3 == 1))
		if ((Channel_3_Active == 0) && (VoltageChangedCh3 == 1))
		{
			DRAWP(158, 4, 6, 10, 0, Channel_Plane3);							// Clear Channel Text	
	//		DRAWP(158, 4, 6, 10, 0, Buffer_Plane3);								// Clear Channel Text	
			DRAWP(155, 0, 68, 18, 0, UI_Plane2);								// Clear Text
	
			DRAWSQUAREBUTTON(154, 1, 14, 16, btnPushed, btnRemove);						// Button
			
		}
		//BF else if ((Channel_3_Active == 1) && (Channel_3_Active_Old == 0) && (VoltageChangedCh3 == 1))
		else if ((Channel_3_Active == 1) && (VoltageChangedCh3 == 1))
		{
			DRAWSQUAREBUTTON(154, 1, 14, 16, btnPushed, btnDraw);						// Button
			TEXTOUTvbig("3", 158, 4, 1, Channel_Plane3);							// Draw Voltage Text		
			//TEXTOUTvbig("3", 158, 4, 1, Buffer_Plane3);							// Draw Voltage Text		
			
			//VoltageChangedCh3 = 1;
			
			//DRAWZERO(3, 0);										// Draw Zero Marker	
		}
		
		//BF if ((Channel_4_Active == 0) && (Channel_4_Active_Old == 1) && (VoltageChangedCh4 == 1))
		if ((Channel_4_Active == 0) && (VoltageChangedCh4 == 1))
		{
			DRAWP(234, 4, 6, 10, 0, Channel_Plane4);							// Clear Channel Text	
			//BF DRAWP(234, 4, 6, 10, 0, Buffer_Plane4);							// Clear Channel Text	
			DRAWP(231, 0, 68, 18, 0, UI_Plane2);								// Clear Text
	
			DRAWSQUAREBUTTON(230, 1, 14, 16, btnPushed, btnRemove);						// Button
			
			//REMOVEZERO(4);										// Remove Zero Marker		
		}
		//BF else if ((Channel_4_Active == 1) && (Channel_4_Active_Old == 0) && (VoltageChangedCh4 == 1))
		else if ((Channel_4_Active == 1) && (VoltageChangedCh4 == 1))
		{
			DRAWSQUAREBUTTON(230, 1, 14, 16, btnPushed, btnDraw);						// Button		
			TEXTOUTvbig("4", 234, 4, 1, Channel_Plane4);							// Draw Voltage Text		
			//TEXTOUTvbig("4", 234, 4, 1, Buffer_Plane4);							// Draw Voltage Text		
			
			//VoltageChangedCh4 = 1;
			
			//DRAWZERO(4, 0);	
		}
    	}			
	
	// Update Statusbar
	if ((Channel_1_Active == 1) && (VoltageChangedCh1 == 1)) DRAWSTATUS(1, 0);					// Draw Voltage Channel 1
	if ((Channel_2_Active == 1) && (VoltageChangedCh2 == 1)) DRAWSTATUS(2, 0);					// Draw Voltage Channel 2
	if (NumberOfChannels > 2)
	{
		if ((Channel_3_Active == 1) && (VoltageChangedCh3 == 1)) DRAWSTATUS(3, 0);				// Draw Voltage Channel 3
		if ((Channel_4_Active == 1) && (VoltageChangedCh4 == 1)) DRAWSTATUS(4, 0);				// Draw Voltage Channel 4		
	}
	
	if (TimebaseChanged == 1) DRAWSTATUS(0, 0);									// Draw Timebase
	if (TimeOffsetChanged == 1) DRAWSTATUS(5, 0);									// Draw Timebase Offset
	if (TriggerLevelChanged == 1) DRAWSTATUS(6, 0);									// Draw Trigger Level
	if (TriggerModeChanged == 1) DRAWSTATUS(7, 0);									// Draw Auto	
	if (TriggerWayChanged == 1) DRAWSTATUS(8, 0);									// Draw Trigger way	

/*	
	if (Selected_Trigger_Source != Selected_Trigger_Source_Old)
	{
		DRAWZEROS();		
	}	
*/	
	MenuTimeCounter = 0;	
}
