#include "hardware_t.h"
#include "tc_vars.h"
#include "flash_t.h"
#include "display_t.h"
//#include "math.h"

// Class Display

//##########################################################################################################################################################
/* BF */
// Build trigonometric tables for FFT
void Display::FFT_BuildTrigoTables(void)
{
	//Speicher auf dem Stack reservieren
	int z,l,i,r,ct,buff; 	//,ar,v, la;
	int d[FFT_Level];	//Number of butterfly levels

	//Struktur vom Typ div_t erzeugen
	div_t s_div;

/*
	Display::DRAWROUNDBUTTON(230, 180, 200, 80, 0, 0);
	Display::TEXTOUTxvbig("Building trigonometric", 247, 192, 1, UI_Plane2);
	Display::TEXTOUTxvbig("tables.", 247, 212, 1, UI_Plane2);
	Display::TEXTOUTxvbig("Please wait a minute.", 247, 232, 1, UI_Plane2);
*/

/*
	// create sin() and cos() table as integer values
	for(i=0;i < FFT_Length;i++)  			
	{
		sin_tab[i] = (int)(FFT_INTFACT * sin(i*2*PI/FFT_Length));
		cos_tab[i] = (int)(FFT_INTFACT * cos(i*2*PI/FFT_Length));
	}
*/	
	// create window and trigonometric tables
	for(i=0,r=0 ;i < FFT_Length; i++)
	{
		sin_tab[i] = iSin1024[r];
		cos_tab[i] = iCos1024[r];

		win_tab[i] = FFT_Window[FFT_Windex][r];
	
		if (FFT_Length == FFT_512)
		r+=2;
		else
		r++;	
	}

	
	//Bit-Reversal- Tabelle bit_rev[0..] anlegen
	for(i=0 ;i < FFT_Length; i++)
	{
		z=i;
		//Dezimal-Index in Dualzahl umwandeln
		for(ct=0 ;ct < FFT_Level; ct++)
		{
			s_div = div(z,2);
			d[ct] = (int)s_div.rem;		// den Rest ins Feld
			z     = (int)s_div.quot;	// den Quotienten nach z
		}
		//Bit-Revers anordnen
		s_div = div(FFT_Level,2);
		for(ct=0 ;ct < (int)s_div.quot; ct++)
		{
			buff  = d[ct];
			d[ct] = d[FFT_Level-ct-1];
			d[FFT_Level-ct-1] = buff;
		}
		
		//Bit-Revers anordnen Zeitoptimiert mit Festwert
//		for(int buff,j=0;j<5;j++) 	        // 5 = FFT_LEVEL/radix  => x=div(FFT_LEVEL,2) => x.quot=5
//		{
//			buff 	     = d[j];
//			d[j] 	     = d[FFT_LEVEL-j-1];
//			d[FFT_LEVEL-j-1] = buff;
//		}

		//Umsortierten Index zurückwandeln in Dezimalzahl
		r = 0;
		for(ct=0 ;ct < FFT_Level; ct++)
		{
			r = r + (1<<ct) * d[ct];
		}
		//umsortierter Index
		bit_rev[i] = r;

	}//Ende Bit-Reversal-Tabelle
/*	
	//Delete popup
	Display::DRAWROUNDBUTTON(230, 180, 200, 80, 0, 1);
	Display::TEXTOUTxvbig("Building trigonometric", 247, 192, 0, UI_Plane2);
	Display::TEXTOUTxvbig("tables.", 247, 212, 0, UI_Plane2);
	Display::TEXTOUTxvbig("Please wait a minute.", 247, 232, 0, UI_Plane2);
*/
/*
	//check the values
	for(i=0;i < FFT_Length;i++)  			// {sin und cos-Tabelle prüfen}
	{
		//printf("SIN(x) = ");floatprintf((float)sin_tab[i]);
		//printf("COS(x) = ");floatprintf((float)cos_tab[i]);printf("\r\n");
		printf("SIN(x) =  %d\r\n",sin_tab[i]);
		printf("COS(x) =  %d\r\n",cos_tab[i]);

	}
	//check the values
//	for(Ii=0 ;Ii < FFT_LENGTH; Ii++)
//	printf("Bit reversal =  %d\r\n",bit_rev[Ii]);

*/

}

//##########################################################################################################################################################
/* BF */
// Calculate fast fourier transformation to get spectral analyzis of time signal. Trigonometric tables have to be build first.
// To get a periodic approximation, the time signal has to be processed with a window function. The simplest case is a 
// rectangle window, which has no side line suppression.
void Display::FFT_CalcSpectrum(unsigned char *Signal, int *Spectrum)
{
	// buffer for real and imaginary part of spectrum - sharing the memory with the signal interpolation
	int *freq_re = (int *)0x008DDD20;
	int *freq_im = (int *)0x008DF490;
	
	//Speicher auf dem Stack reservieren
	int l=0,i=0,j=0,r=0,ar=0,v=0;
	int Fz1=0,Fz2=0,Fz3=0,Fz4=0,Fxr=0,Fxi=0,Fyr=0,Fyi=0;
	int ScaleShiftFactor;  
	
	double logval;
    	//printf("starting fft\r\n");
	
	// set variable scaling
	if (FFT_Mode == FFT_PWSP || FFT_Mode == FFT_PWDN)
	FFT_Scale = 3;
	else
	FFT_Scale = 2;

	ScaleShiftFactor = FFT_Level - FFT_Scale;

	//-----------------------------------------------
	// get time signal and make a window overlay
	//-----------------------------------------------

	//sort timevalues bitreversal
	for(i=0;i<FFT_Length;i++)
	{
		if(FFT_Windex == -1)
		freq_re[i] = (int)(Signal[bit_rev[i]] - ADC_ZERO);					// no window overlay -> rectangle
		else
		freq_re[i] =  (win_tab[bit_rev[i]] * (int)(Signal[bit_rev[i]] - ADC_ZERO)) >> 13;	// window overlay

		freq_im[i] = 0;

		//printf("freq_re[%d] = ",Ii);floatprintf(freq_re[Ii]);
		//printf("     freq_im[%d] = ",Ii);floatprintf(freq_im[Ii]);printf("\r\n");
	}

	//-----------------------------------------------
	//             calculate FFT
	//-----------------------------------------------

	//calculate complex spectral values (butterflies)
	for(l=1;(l<=FFT_Level) && !UI_request;l++)	// level loop
	{
		for(i=0;i<=((1<<(FFT_Level-l))-1) && !UI_request;i++)
		{
			for(j=0;j<=((1<<(l-1))-1);j++)
			{
				r = j + i* (1 << l);
				ar= j * (FFT_Length >> l);
				v = 1 << (l-1);
				
				Fz1 = (cos_tab[ar] * freq_re[r+v]) >> 10;	//rescale from integer conversion
				Fz2 = (cos_tab[ar] * freq_im[r+v]) >> 10;	//rescale from integer conversion
				Fz3 = (sin_tab[ar] * freq_re[r+v]) >> 10;	//rescale from integer conversion
				Fz4 = (sin_tab[ar] * freq_im[r+v]) >> 10;	//rescale from integer conversion
				Fxr = freq_re[r] + Fz1 + Fz4;
				Fxi = freq_im[r] - Fz3 + Fz2;
				Fyr = freq_re[r] - Fz1 - Fz4;
				Fyi = freq_im[r] + Fz3 - Fz2;
				freq_re[r] = Fxr;
				freq_im[r] = Fxi;
				freq_re[r+v] = Fyr;
				freq_im[r+v] = Fyi;
			}
		}
	}


	//get half of the spectrum because spectrum is mirrored at the middle and calculate output
	for (i=0,j=0;i < (FFT_Length>>1)  && !UI_request;i++)
	{
		//downscale depending on FFT length
		freq_re[i] = freq_re[i] >> ScaleShiftFactor;
		freq_im[i] = freq_im[i] >> ScaleShiftFactor;

		switch(FFT_Mode)
		{
			case FFT_MAGN: 	// linear magnitude
			{
				//calc total value via power of two
				v = (freq_re[i]*freq_re[i]) + (freq_im[i]*freq_im[i]);	// v = Re² + IM²

				// and square root
				for( l = 0,r = 1 ; l <= SQRT_PREC; l++)			// r = sqrt(v)
				r = (r + (v / r)) >> 1;
				
				break;
			}
			
			case FFT_PHAS: 	// linear phase
			{
				r = 10;
				//phase = arctan(imag / real)
				break;
			}

			case FFT_REAL: 	// linear real part
			{
				r = freq_re[i];
				
				break;
			}

			case FFT_IMAG: 	// linear imaginary part
			{
				r = freq_im[i];
				
				break;
			}

			case FFT_REIM: 	// linear real + imaginary
			{
				r = freq_re[i] + freq_im[i];
				
				break;
			}

			case FFT_PWSP: 	// power spectrum in dBm
			{
				//calc total value via power of two
				v = (freq_re[i]*freq_re[i]) + (freq_im[i]*freq_im[i]);	// v = Re² + IM²

				// and square root
				for( l = 0,r = 1 ; l <= SQRT_PREC; l++)			// r = sqrt(v)
				r = (r + (v / r)) >> 1;

				//limiter
				if (r < 0) r = 0;
				if (r > 480) r = 480;

				// calculate logarithm
				r = (int)iLog10[r];

/*
				//limiter
				if (r < 0) r = 0;
				if (r > 480) r = 480;

				// calculate power spectrum
				// ps = 20 * log10(magn / ref)		ref = 0.316 V = 323 / 1024
				
				r = (r * 323) >> 10;			//magn / ref
				
				r = (int)iLog10[r];			//20 * log10(magn / ref)
*/				
				break;			}

			case FFT_PWDN: 	// power density spectrum in dBm
			{
				// calculate power density spectrum
				// pd = ps - 10 * log10(ENBW * df)

				r = 10;
				
				break;
			}

		}

		//printf("freq_re[%d] = %d    ",i,freq_re[i]);
		//printf("freq_im[%d] = %d    ",i,freq_im[i]);	

		
		//printf("v = Re² + IM² = %d    ",v);
		

		//printf("r = sqrt(v) = %d",r);
		
		// scale to display size
		//r = r << 2;
		//printf("scale r = %d\r\n",r);	

		
		//map to 512 or 1024 values
		if(FFT_Length == FFT_1024 )
			Spectrum[i] = r;
		else
		{ 
			Spectrum[j] = r;
			Spectrum[j+1] = r;
			j+=2;
		 }	
		
	}

    	//printf("finished fft\r\n");


}

//##########################################################################################################################################################
/* BF */
// draw informations about FFT on right side of the screen
void Display::FFT_DrawStatus(unsigned char clear)
{

	float lValue = 0;
	int lTxtOffs = GRID_XOFFS_FFT + GRID_WIDTH_FFT + 9;


		//if (MenuPopupActive > -1)
		//nr_delay(1000);

	if(clear && FFT_Layout)
	{	
		//delete FFT-status area
		int xa = 19 * 20;		// Ypos * 20
		int xo = 16;	//512 / 32;	// Xpos / 32
	
		for (int Ct = 0; Ct < 400 ; Ct++)	// Ysize = 110
		{			
			*(UI_Plane1 + xo + xa)     = 0;	// Xsize + 1 (+32), + 2 (+ 2*32)...
			*(UI_Plane1 + xo + xa + 1) = 0;
			*(UI_Plane1 + xo + xa + 2) = 0;
			*(UI_Plane1 + xo + xa + 3) = 0;
	
			*(UI_Plane2 + xo + xa)     = 0;	// Xsize + 1 , + 2 ...
			*(UI_Plane2 + xo + xa + 1) = 0;
			*(UI_Plane2 + xo + xa + 2) = 0;
			*(UI_Plane2 + xo + xa + 3) = 0;
	
			*(UI_Plane3 + xo + xa)     = 0;	// Xsize + 1 , + 2 ...
			*(UI_Plane3 + xo + xa + 1) = 0;
			*(UI_Plane3 + xo + xa + 2) = 0;
			*(UI_Plane3 + xo + xa + 3) = 0;
	
			*(UI_Plane4 + xo + xa)     = 0;	// Xsize + 1 , + 2 ...
			*(UI_Plane4 + xo + xa + 1) = 0;
			*(UI_Plane4 + xo + xa + 2) = 0;
			*(UI_Plane4 + xo + xa + 3) = 0;
	
			*(UI_Plane5 + xo + xa)     = 0;	// Xsize + 1 , + 2 ...
			*(UI_Plane5 + xo + xa + 1) = 0;
			*(UI_Plane5 + xo + xa + 2) = 0;
			*(UI_Plane5 + xo + xa + 3) = 0;
	
			xa += 20;
		}
		return;
	}


//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

	if(FFT_Mode == FFT_OFF) return;

	
	// SET FFT cursor data display
	if (FFT_Mode == FFT_PWSP || FFT_Mode == FFT_PWDN)
	{
		FFT_Cursor_Delta_Y.Init(0.00000001, 0.00000001, 10, 0.000000005, 2, 4, "", "", "dBm");
		FFT_Cursor_Horizontal_Pos_1_Str.Init(0.00000001, 0.00000001, 10, 0.000000005, 2, 4, "", "", "dBm");
		FFT_Cursor_Horizontal_Pos_2_Str.Init(0.00000001, 0.00000001, 10, 0.000000005, 2, 4, "", "", "dBm");
	}
	else
	{
		FFT_Cursor_Delta_Y.Init(0.00000001, 0.00000001, 10, 0.000000005, 2, 4, "", "", "V");
		FFT_Cursor_Horizontal_Pos_1_Str.Init(0.00000001, 0.00000001, 10, 0.000000005, 2, 4, "", "", "V");
		FFT_Cursor_Horizontal_Pos_2_Str.Init(0.00000001, 0.00000001, 10, 0.000000005, 2, 4, "", "", "V");
	}


//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

			
/*
	//TEXTOUTxvbig(&string[0], GRID_XOFFS_FFT + GRID_WIDTH_FFT + 10, 40, 1,  Grid_Plane);	

	if (FFT_Mode == FFT_LOG)
	{
		TEXTOUTxvbig(" 0dB",    GRID_XOFFS_FFT + GRID_WIDTH_FFT + 5, 25, 1,  Grid_Plane);	
		TEXTOUTxvbig("-10dB",  GRID_XOFFS_FFT + GRID_WIDTH_FFT + 5, 95, 1,  Grid_Plane);	
		TEXTOUTxvbig("-20dB",  GRID_XOFFS_FFT + GRID_WIDTH_FFT + 5, 175, 1,  Grid_Plane);	
		TEXTOUTxvbig("-30dB",  GRID_XOFFS_FFT + GRID_WIDTH_FFT + 5, 255, 1,  Grid_Plane);
		TEXTOUTxvbig("-40dB",  GRID_XOFFS_FFT + GRID_WIDTH_FFT + 5, 335, 1,  Grid_Plane);	
	}
*/


	
/*
	int xa = 70 * 20;		// Ypos * 20
	int xo = 17;	//544 / 32;	// Xpos / 32

	for (int Ct = 0; Ct < 110 ; Ct++)	// Ysize = 110
	{			
		*(Grid_Plane + xo + xa)     = 0;// Xsize + 1 , +2 ...
		*(Grid_Plane + xo + xa + 1) = 0;
//		*(Grid_Plane + xo + xa + 2) = 0;
		xa += 20;
	}
*/

//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

	if(FFT_Layout)	// display variant with buttons
	{
		Display::DRAWROUNDBUTTON(516, 19, 124, 120, 0, 0);
		Display::DRAWROUNDBUTTON(516, 144, 124, 120, 0, 0);
		Display::DRAWROUNDBUTTON(516, 270, 124, 120, 0, 0);
	
		DRAWLINEP(519, 49, 638, 49, 1, UI_Plane2);	// time under line
		DRAWLINEP(519, 174, 638, 174, 1, UI_Plane2);	// frequency under line
		DRAWLINEP(519, 300, 638, 300, 1, UI_Plane2);	// magnitude under line
	
	
		//--------------------------------------
		//	time status area
		//--------------------------------------
		DRAWP(546, 55, 92, 70, 0, UI_Plane2);	// delete 
	
		// time header
		TEXTOUTxvbig("Time Source", lTxtOffs + 10, 28, 1, UI_Plane2);
	
		// source channel
		TEXTOUTxvbig("Ch :", lTxtOffs, 60, 1, UI_Plane2);	
		TEXTOUTxvbig(&MenuPopupText[29][MenuStatus[27][0] - 137][0], lTxtOffs + 35, 60, 1, UI_Plane2);
	
		// FFT length
		TEXTOUTxvbig("Len:", lTxtOffs, 85, 1, UI_Plane2);
		TEXTOUTxvbig(&MenuPopupText[32][MenuStatus[27][3] - 82][0], lTxtOffs + 35, 85, 1, UI_Plane2);
		TEXTOUTxvbig("pts", lTxtOffs + 70, 85, 1, UI_Plane2);

		// time window
		TEXTOUTxvbig("Win:", lTxtOffs, 110, 1, UI_Plane2);	
		TEXTOUTxvbig(&MenuPopupText[30][MenuStatus[27][1] - 192][0], lTxtOffs + 35, 110, 1, UI_Plane2);
	
	
		//--------------------------------------
		//	frequency status area
		//--------------------------------------
		DRAWP(546, 180, 92, 80, 0, UI_Plane2);	//delete 
	
		// frequency header
		TEXTOUTxvbig("Frequency", lTxtOffs + 21, 153, 1, UI_Plane2);	
	
		// calculate delta frequency
		lValue = (float)SampleRate[Display_Timebase] / (float)FFT_Length;	
		// start value , min value, max value, stepwidth , places afterdot, max places, Text, PreText, Unit
		FFT_Status_Str.Init(lValue, 1, 1, 1, 2, 5, "", "df : ", "Hz");
		TEXTOUTxvbig(FFT_Status_Str.Read_Text(), lTxtOffs, 185, 1, UI_Plane2);	
	
		// calculate and display frequency per div
		lValue *= (float)(FFT_Length >> 4);
		FFT_Status_Str.Init(lValue, 1, 1, 1, 2, 5, "", "div: ", "Hz");
		TEXTOUTxvbig(FFT_Status_Str.Read_Text(), lTxtOffs, 210, 1, UI_Plane2);	
	
		// calculate and display fmax 
		lValue = (float)(SampleRate[Display_Timebase] >> 1);
		FFT_Status_Str.Init(lValue, 1, 1, 1, 2, 5, "", "fN : ", "Hz");
		TEXTOUTxvbig(FFT_Status_Str.Read_Text(), lTxtOffs, 235, 1, UI_Plane2);	
	
	
	
		//--------------------------------------
		//	magnitude status area
		//--------------------------------------
		DRAWP(lTxtOffs+20, 278, 90, 18, 0, UI_Plane2);	//delete header
		DRAWP(lTxtOffs, 305, 110, 80, 0, UI_Plane2);	//delete lower part
	
		// magnitude header
		//TEXTOUTxvbig("Magnitude", lTxtOffs + 21, 279, 1, UI_Plane2);	
		TEXTOUTxvbig(&MenuPopupText[31][FFT_Mode][0], lTxtOffs+21, 279, 1, UI_Plane2);	
		// FFT mode
		//TEXTOUTxvbig(&MenuPopupText[31][FFT_Mode][0], lTxtOffs, 311, 1, UI_Plane2);	

		// linear - logarithmic
		if (FFT_Mode == FFT_PWSP || FFT_Mode == FFT_PWDN)
		TEXTOUTxvbig("Mag: log (dBm)", lTxtOffs, 311, 1, UI_Plane2);	
		else
		TEXTOUTxvbig("Mag: lin (Volt)", lTxtOffs, 311, 1, UI_Plane2);	

		//TEXTOUTxvbig("???: ????????", lTxtOffs, 336, 1, UI_Plane2);	

		//TEXTOUTxvbig("???: ????????", lTxtOffs, 362, 1, UI_Plane2);	

	}
	
//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
	else	// "black" display variant
	{
		//draw frame
		DRAWLINEP(516, 19, 638, 19, 1, Grid_Plane);	// time top line
		DRAWLINEP(516, 59, 638, 59, 1, Grid_Plane);	// time under line
	
		DRAWLINEP(516, 154, 638, 154, 1, Grid_Plane);	// frequency top line
		DRAWLINEP(516, 194, 638, 194, 1, Grid_Plane);	// frequency under line
	
		DRAWLINEP(516, 290, 638, 290, 1, Grid_Plane);	// magnitude top line
		DRAWLINEP(516, 330, 638, 330, 1, Grid_Plane);	// magnitude under line
	
		DRAWLINEP(638, 19, 638, 421, 1, Grid_Plane);	//right line
		DRAWLINEP(516, 421, 638, 421, 1, Grid_Plane);	//bottom line
	
		//--------------------------------------
		//delete time status area
		//--------------------------------------
		DRAWP(546, 70, 92, 70, 0, Grid_Plane);	
	
		// time header
		TEXTOUTxvbig("Time Source", lTxtOffs + 10, 33, 1, Grid_Plane);	
	
		// source channel
		TEXTOUTxvbig("Ch :", lTxtOffs, 75, 1, Grid_Plane);	
		TEXTOUTxvbig(&MenuPopupText[29][MenuStatus[27][0] - 137][0], lTxtOffs + 35, 75, 1, Grid_Plane);
	
		// FFT length
		TEXTOUTxvbig("Len:", lTxtOffs, 100, 1, Grid_Plane);
		TEXTOUTxvbig(&MenuPopupText[32][MenuStatus[27][3] - 82][0], lTxtOffs + 35, 100, 1, Grid_Plane);
		TEXTOUTxvbig("pts", lTxtOffs + 70, 100, 1, Grid_Plane);

		// time window
		TEXTOUTxvbig("Win:", lTxtOffs, 125, 1, Grid_Plane);	
		TEXTOUTxvbig(&MenuPopupText[30][MenuStatus[27][1] - 192][0], lTxtOffs + 35, 125, 1, Grid_Plane);



	
		//--------------------------------------
		//delete frequency status area
		//--------------------------------------
		DRAWP(546, 205, 92, 80, 0, Grid_Plane);	
	
	
		// frequency header
		TEXTOUTxvbig("Frequency", lTxtOffs + 21, 168, 1, Grid_Plane);	
	
		// calculate delta frequency
		lValue = (float)SampleRate[Display_Timebase] / (float)FFT_Length;	
		// start value , min value, max value, stepwidth , places afterdot, max places, Text, PreText, Unit
		FFT_Status_Str.Init(lValue, 1, 1, 1, 2, 5, "", "df : ", "Hz");
		TEXTOUTxvbig(FFT_Status_Str.Read_Text(), lTxtOffs, 210, 1, Grid_Plane);	
	
		// calculate and display frequency per div
		lValue *= (float)(FFT_Length >> 4);
		FFT_Status_Str.Init(lValue, 1, 1, 1, 2, 5, "", "div: ", "Hz");
		TEXTOUTxvbig(FFT_Status_Str.Read_Text(), lTxtOffs, 235, 1, Grid_Plane);	
	
		// calculate and display fmax 
		lValue = (float)(SampleRate[Display_Timebase] >> 1);
		FFT_Status_Str.Init(lValue, 1, 1, 1, 2, 5, "", "fN : ", "Hz");
		TEXTOUTxvbig(FFT_Status_Str.Read_Text(), lTxtOffs, 260, 1, Grid_Plane);	
	
		//--------------------------------------
		//delete magnitude status area
		//--------------------------------------
		DRAWP(lTxtOffs+20, 302, 90, 18, 0, Grid_Plane);		//delete header
		DRAWP(lTxtOffs, 340, 92, 80, 0, Grid_Plane);		//delete lower part
	
		// magnitude header
		//TEXTOUTxvbig("Magnitude", lTxtOffs + 21, 304, 1, Grid_Plane);	
		TEXTOUTxvbig(&MenuPopupText[31][FFT_Mode][0], lTxtOffs+21, 304, 1, Grid_Plane);	

		// linear - logarithmic
		if (FFT_Mode == FFT_PWSP || FFT_Mode == FFT_PWDN)
		TEXTOUTxvbig("Mag: log (dBm)", lTxtOffs, 346, 1, Grid_Plane);	
		else
		TEXTOUTxvbig("Mag: lin (Volt)", lTxtOffs, 346, 1, Grid_Plane);
	
		//TEXTOUTxvbig("???: ????????", lTxtOffs, 371, 1, Grid_Plane);	
		//TEXTOUTxvbig("???: ????????", lTxtOffs, 396, 1, Grid_Plane);	


	}

}

//##########################################################################################################################################################
//BF calculate cursor data for FFT
void Display::FFT_CalcCursorData(void)  								// channel = source im cursor menu
{
	int buf_int = 0;	
		
	//BF del char doit = 0;	
	int volt = 0;		
	int zero = 0;		
	int probe_idx = 0;
/*
	double vfactor  = 0;	
	double vfactor2 = 0;	
	double voltoff  = 0;		
*/
	float vfactor  = 0;	
	float probefact = 0;	
	float voltoff  = 0;		
	
	int time = 0;
/*
	double tfactor = 0;
	double timeoff = 0;			
*/	
	//float tfactor = 0;
	//float timeoff = 0;	

	float frequ1, frequ2;
	float dfrequ;

	char BufferText[10];
	
	int channel = MenuStatus[MENU_FFT][0] - 137;

	//printf("calculating FFT-cursor\r\n");
			
	switch(channel)
	{
	 	case 1:
		{				
			volt = Selected_Voltage_CH1;
			probe_idx = MenuStatus[1][3];
			break;
		}
		case 2:
		{				
			volt = Selected_Voltage_CH2;
			probe_idx = MenuStatus[2][3];
			break;
		}
		case 3:
		{				
			volt = Selected_Voltage_CH3;
			probe_idx = MenuStatus[3][3];
			break;
		}
		case 4:
		{				
			volt = Selected_Voltage_CH4;
			probe_idx = MenuStatus[4][3];
			break;	
		}
	}
	
	zero = GRID_HEIGHT_FFT + 1;

	if (FFT_Mode == FFT_PWSP || FFT_Mode == FFT_PWDN)
	{
		vfactor = 0.125;	// * probefact
	}
	else	//linear magnitude
	{
		switch(volt)
		{
			case 0 : vfactor = 0.00002; break;
			case 1 : vfactor = 0.00004; break;
			case 2 : vfactor = 0.0001; break;
			case 3 : vfactor = 0.0002; break;
			case 4 : vfactor = 0.0004; break;
			case 5 : vfactor = 0.001; break;
			case 6 : vfactor = 0.002; break;
			case 7 : vfactor = 0.004; break;
			case 8 : vfactor = 0.01; break;
			case 9 : vfactor = 0.02; break;
			case 10 : vfactor = 0.04; break;
			case 11 : vfactor = 0.1; break;
		}
	
		switch(probe_idx)	// probe predivision
		{
	//		case 101: probefact = 0.1; break;	//not used
	//		case 102: probefact = 0.2; break;	//not used
	//		case 103: probefact = 0.5; break;	//not used
			case 104: probefact = 1; break;
			case 105: probefact = 2; break;
			case 106: probefact = 5; break;
			case 107: probefact = 10; break;
			case 108: probefact = 20; break;
			case 109: probefact = 50; break;
			case 110: probefact = 100; break;
			case 111: probefact = 200; break;
			case 112: probefact = 500; break;
			case 113: probefact = 1000; break;
		}	
	
		vfactor = vfactor * probefact / 2;	// magnitude factor
	
	}

	buf_int = zero - Cursor_Horizontal_Position_Real1;
	//printf("Horizontal cursor 1 = %d",buf_int);	
	//printf(" | ");

	voltoff1 =  (float)buf_int * vfactor; 
	
	// display horizontal cursor Y1 data
	FFT_Cursor_Horizontal_Pos_1_Str.Write_Value(voltoff1);
	FFT_Cursor_Horizontal_Pos_1_Str.RenderText();
		
	
	buf_int = zero - Cursor_Horizontal_Position_Real2;	
	//printf("Horizontal cursor 2 = %d\r\n",buf_int);	

	voltoff2 =  (float)buf_int * vfactor; 
	
	// display horizontal cursor Y2 data
	FFT_Cursor_Horizontal_Pos_2_Str.Write_Value(voltoff2);
	FFT_Cursor_Horizontal_Pos_2_Str.RenderText();	
	

	if (voltoff1 > voltoff2)
	{ voltoff = voltoff1 - voltoff2; }
	else
	{ voltoff = voltoff2 - voltoff1; }
	
	if (voltoff < 0) voltoff = -voltoff;

		
	switch(channel)
	{
		case 1: sprintf(BufferText, "@Y [ = "); break;
		case 2: sprintf(BufferText, "@Y \\ = "); break;
		case 3: sprintf(BufferText, "@Y ] = "); break;
		case 4: sprintf(BufferText, "@Y ^ = "); break;
		//case 5: sprintf(BufferText, "@Y(M) = "); break;
	}

	// display delta cursor Y data
	FFT_Cursor_Delta_Y.Write_PreText(BufferText);
	FFT_Cursor_Delta_Y.Write_Value(voltoff);
	FFT_Cursor_Delta_Y.RenderText();				
	


	//------------------------------------------------------------------------------------
	//		Vertical cursors - frequency
	//------------------------------------------------------------------------------------
	// limiter
	if(Cursor_Vertical_Position_Real1 > GRID_WIDTH_FFT + 1) Cursor_Vertical_Position_Real1 = GRID_WIDTH_FFT + 1;
	if(Cursor_Vertical_Position_Real2 > GRID_WIDTH_FFT + 1) Cursor_Vertical_Position_Real2 = GRID_WIDTH_FFT + 1;
	//printf("Vertical cursor 1 = %d",Cursor_Vertical_Position_Real1);
	//printf(" | ");
	//printf("Vertical cursor 2 = %d\r\n",Cursor_Vertical_Position_Real2);


	// calculate delta frequency
	dfrequ =  (float)SampleRate[Display_Timebase] / 1024.0;	//(float)FFT_Length;	
	
	// calculate frequency of cursor 1 position
	frequ1 = dfrequ * (float)(Cursor_Vertical_Position_Real1 - 1);

	// display vertical cursor X1 data 
	FFT_Cursor_Vertical_Pos_1_Str.Write_Value(frequ1);
	FFT_Cursor_Vertical_Pos_1_Str.RenderText();	
	//printf("freq1 = ");floatprintf(frequ1);printf("\r\n");	

	// calculate frequency of cursor 2 position
	frequ2 = dfrequ * (float)(Cursor_Vertical_Position_Real2 - 1);

	// display vertical cursor X2 data 
	FFT_Cursor_Vertical_Pos_2_Str.Write_Value(frequ2);	
	FFT_Cursor_Vertical_Pos_2_Str.RenderText();	
	//printf("freq2 = ");floatprintf(frequ2);printf("\r\n");	
	
	// calculate delta cursor 1 - cursor 2
	if (frequ1 > frequ2) dfrequ = frequ1 - frequ2;
	else dfrequ = frequ2 - frequ1;
	if (dfrequ < 0) dfrequ = -dfrequ;
	
	// display delta
	FFT_Cursor_Delta_X.Write_Value(dfrequ);	
	FFT_Cursor_Delta_X.RenderText();
	
	// calculate and display 1/delta
	dfrequ = 1 / dfrequ;
	FFT_Cursor_Delta_1X.Write_Value(dfrequ);
	FFT_Cursor_Delta_1X.RenderText();
	
}
